@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: classdef.h,v 1.39 2001/03/19 19:27:39 root Exp $
+ * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,22 +18,20 @@
 #ifndef CLASSDEF_H
 #define CLASSDEF_H
 
-#include "qtbc.h"
 #include <qlist.h>
 #include <qdict.h>
 #include <qptrdict.h>
-#include <qstrlist.h>
 
-#include "util.h"
-#include "memberlist.h"
 #include "definition.h"
-#include "sortdict.h"
 
+class MemberDef;
+class MemberList;
 class MemberDict;
 class ClassList;
 class ClassSDict;
 class OutputList;
 class FileDef;
+class FileList;
 class BaseClassList;
 class NamespaceDef;
 class MemberDef;
@@ -47,23 +45,28 @@ class GroupDef;
 class StringDict;
 struct IncludeInfo;
 class ClassDefImpl;
+class ArgumentList;
+class FTextStream;
 
-/** A model of a compound symbol.
+/** A class representing of a compound symbol.
  *
- *  A compound can be a class, struct, union, interface, or exception.
+ *  A compound can be a class, struct, union, interface, service, singleton,
+ *  or exception.
  *  \note This class should be renamed to CompoundDef
  */
 class ClassDef : public Definition
 {
   public:
     /** The various compound types */
-    enum CompoundType { Class, //=Entry::CLASS_SEC, 
-                        Struct, //=Entry::STRUCT_SEC, 
-                        Union, //=Entry::UNION_SEC,
+    enum CompoundType { Class,     //=Entry::CLASS_SEC, 
+                        Struct,    //=Entry::STRUCT_SEC, 
+                        Union,     //=Entry::UNION_SEC,
                         Interface, //=Entry::INTERFACE_SEC,
-                        Protocol, //=Entry::PROTOCOL_SEC,
-                        Category, //=Entry::CATEGORY_SEC,
-                        Exception //=Entry::EXCEPTION_SEC
+                        Protocol,  //=Entry::PROTOCOL_SEC,
+                        Category,  //=Entry::CATEGORY_SEC,
+                        Exception, //=Entry::EXCEPTION_SEC
+                        Service,   //=Entry::CLASS_SEC
+                        Singleton, //=Entry::CLASS_SEC
                       };
 
     /** Creates a new compound definition.
@@ -71,6 +74,8 @@ class ClassDef : public Definition
      *                   found.
      *  \param startLine line number where the definition of this compound
      *                   starts.
+     *  \param startColumn column number where the definition of this compound
+     *                   starts.
      *  \param name      the name of this compound (including scope)
      *  \param ct        the kind of Compound
      *  \param ref       the tag file from which this compound is extracted
@@ -84,7 +89,7 @@ class ClassDef : public Definition
      *                    I didn't add this to CompoundType to avoid having
      *                    to adapt all translators.
      */
-    ClassDef(const char *fileName,int startLine,
+    ClassDef(const char *fileName,int startLine,int startColumn,
              const char *name,CompoundType ct,
              const char *ref=0,const char *fName=0,
              bool isSymbol=TRUE,bool isJavaEnum=FALSE);
@@ -121,6 +126,9 @@ class ClassDef : public Definition
     /** returns TRUE if this class has documentation */
     bool hasDocumentation() const;
 
+    /** returns TRUE if this class has a non-empty detailed description */
+    bool hasDetailedDescription() const;
+
     /** Returns the name as it is appears in the documentation */
     QCString displayName(bool includeScope=TRUE) const;
 
@@ -161,6 +169,9 @@ class ClassDef : public Definition
     /** the class is visible in a class diagram, or class hierarchy */
     bool isVisibleInHierarchy();
     
+    /** show this class in the declaration section of its parent? */
+    bool visibleInParentsDeclList() const;
+
     /** Returns the template arguments of this class 
      *  Will return 0 if not applicable.
      */
@@ -187,6 +198,11 @@ class ClassDef : public Definition
      */
     bool isBaseClass(ClassDef *bcd,bool followInstances,int level=0);
 
+    /** Returns TRUE iff \a bcd is a direct or indirect sub class of this
+     *  class.
+     */
+    bool isSubClass(ClassDef *bcd,int level=0);
+
     /** returns TRUE iff \a md is a member of this class or of the
      *  the public/protected members of a base class 
      */
@@ -231,7 +247,7 @@ class ClassDef : public Definition
     void getTemplateParameterLists(QList<ArgumentList> &lists) const;
 
     QCString qualifiedNameWithTemplateParameters(
-        QList<ArgumentList> *actualParams=0) const;
+        QList<ArgumentList> *actualParams=0,int *actualParamIndex=0) const;
 
     /** Returns TRUE if there is at least one pure virtual member in this
      *  class.
@@ -244,6 +260,21 @@ class ClassDef : public Definition
     /** Returns TRUE if this class is implemented in C# */
     bool isCSharp() const;
 
+    /** Returns TRUE if this class is marked as final */
+    bool isFinal() const;
+
+    /** Returns TRUE if this class is marked as sealed */
+    bool isSealed() const;
+
+    /** Returns TRUE if this class is marked as published */
+    bool isPublished() const;
+
+    /** Returns TRUE if this class represents an Objective-C 2.0 extension (nameless category) */
+    bool isExtension() const;
+
+    /** Returns TRUE if this class represents a forward declaration of a template class */
+    bool isForwardDeclared() const;
+
     /** Returns the class of which this is a category (Objective-C only) */
     ClassDef *categoryOf() const;
 
@@ -253,7 +284,7 @@ class ClassDef : public Definition
     QCString className() const;
 
     /** Returns the members in the list identified by \a lt */
-    MemberList *getMemberList(MemberList::ListType lt);
+    MemberList *getMemberList(MemberListType lt);
 
     /** Returns the list containing the list of members sorted per type */
     const QList<MemberList> &getMemberLists() const;
@@ -279,6 +310,21 @@ class ClassDef : public Definition
 
     bool isJavaEnum() const;
 
+    bool isGeneric() const;
+    bool isAnonymous() const;
+    const ClassSDict *innerClasses() const;
+    QCString title() const;
+
+    QCString generatedFromFiles() const;
+    const FileList &usedFiles() const;
+
+    const ArgumentList *typeConstraints() const;
+    const ExampleSDict *exampleList() const;
+    bool hasExamples() const;
+    QCString getMemberListFileName() const;
+    bool subGrouping() const;
+
+
     //-----------------------------------------------------------------------------------
     // --- setters ----
     //-----------------------------------------------------------------------------------
@@ -287,7 +333,7 @@ class ClassDef : public Definition
     void insertSubClass(ClassDef *,Protection p,Specifier s,const char *t=0);
     void setIncludeFile(FileDef *fd,const char *incName,bool local,bool force); 
     void insertMember(MemberDef *);
-    void insertUsedFile(const char *);
+    void insertUsedFile(FileDef *);
     bool addExample(const char *anchor,const char *name, const char *file);
     void mergeCategory(ClassDef *category);
     void setNamespace(NamespaceDef *nd);
@@ -296,13 +342,14 @@ class ClassDef : public Definition
     void setProtection(Protection p);
     void setGroupDefForAllMembers(GroupDef *g,Grouping::GroupPri_t pri,const QCString &fileName,int startLine,bool hasDocs);
     void addInnerCompound(Definition *d);
-    ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,
+    ClassDef *insertTemplateInstance(const QCString &fileName,int startLine,int startColumn,
                                 const QCString &templSpec,bool &freshInstance);
     void addUsedClass(ClassDef *cd,const char *accessName,Protection prot);
     void addUsedByClass(ClassDef *cd,const char *accessName,Protection prot);
     void setIsStatic(bool b);
     void setCompoundType(CompoundType t);
     void setClassName(const char *name);
+    void setClassSpecifier(uint64 spec);
 
     void setTemplateArguments(ArgumentList *al);
     void setTemplateBaseClassNames(QDict<int> *templateNames);
@@ -315,6 +362,7 @@ class ClassDef : public Definition
 
     void addTaggedInnerClass(ClassDef *cd);
     void setTagLessReference(ClassDef *cd);
+    void setName(const char *name);
 
     //-----------------------------------------------------------------------------------
     // --- actions ----
@@ -335,41 +383,40 @@ class ClassDef : public Definition
                           ClassDef *inheritedFrom,const char *inheritId);
     void writeQuickMemberLinks(OutputList &ol,MemberDef *md) const;
     void writeSummaryLinks(OutputList &ol);
-    void reclassifyMember(MemberDef *md,MemberDef::MemberType t);
+    void reclassifyMember(MemberDef *md,MemberType t);
     void writeInlineDocumentation(OutputList &ol);
     void writeDeclarationLink(OutputList &ol,bool &found,
                               const char *header,bool localNames);
     void removeMemberFromLists(MemberDef *md);
-    void addGroupedInheritedMembers(OutputList &ol,MemberList::ListType lt,
+    void addGroupedInheritedMembers(OutputList &ol,MemberListType lt,
                               ClassDef *inheritedFrom,const QCString &inheritId);
-    int countMembersIncludingGrouped(MemberList::ListType lt,ClassDef *inheritedFrom,bool additional);
+    int countMembersIncludingGrouped(MemberListType lt,ClassDef *inheritedFrom,bool additional);
+    int countInheritanceNodes();
+    void writeTagFile(FTextStream &);
     
     bool visited;
 
   protected:
     void addUsedInterfaceClasses(MemberDef *md,const char *typeStr);
-    bool hasExamples();
     bool hasNonReferenceSuperClass();
     void showUsedFiles(OutputList &ol);
 
   private: 
-    void writeTagFileMarker();
     void writeDocumentationContents(OutputList &ol,const QCString &pageTitle);
     void internalInsertMember(MemberDef *md,Protection prot,bool addToAllList);
-    QCString getMemberListFileName() const;
-    void addMemberToList(MemberList::ListType lt,MemberDef *md,bool isBrief);
-    MemberList *createMemberList(MemberList::ListType lt);
-    void writeInheritedMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title,ClassDef *inheritedFrom,bool invert,QPtrDict<void> *visitedClasses);
-    void writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title,
-                                 const char *subTitle=0,bool showInline=FALSE,ClassDef *inheritedFrom=0,int lt2=-1,QPtrDict<void> *visitedClasses=0);
-    void writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title,bool showInline=FALSE);
-    void writeSimpleMemberDocumentation(OutputList &ol,MemberList::ListType lt);
-    void writePlainMemberDeclaration(OutputList &ol,MemberList::ListType lt,bool inGroup,ClassDef *inheritedFrom,const char *inheritId);
+    void addMemberToList(MemberListType lt,MemberDef *md,bool isBrief);
+    MemberList *createMemberList(MemberListType lt);
+    void writeInheritedMemberDeclarations(OutputList &ol,MemberListType lt,int lt2,const QCString &title,ClassDef *inheritedFrom,bool invert,bool showAlways,QPtrDict<void> *visitedClasses);
+    void writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title,
+                                 const char *subTitle=0,bool showInline=FALSE,ClassDef *inheritedFrom=0,int lt2=-1,bool invert=FALSE,bool showAlways=FALSE,QPtrDict<void> *visitedClasses=0);
+    void writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title,bool showInline=FALSE);
+    void writeSimpleMemberDocumentation(OutputList &ol,MemberListType lt);
+    void writePlainMemberDeclaration(OutputList &ol,MemberListType lt,bool inGroup,ClassDef *inheritedFrom,const char *inheritId);
     void writeBriefDescription(OutputList &ol,bool exampleFlag);
     void writeDetailedDescription(OutputList &ol,const QCString &pageType,bool exampleFlag,
                                   const QCString &title,const QCString &anchor=QCString());
     void writeIncludeFiles(OutputList &ol);
-    void writeAllMembersLink(OutputList &ol);
+    //void writeAllMembersLink(OutputList &ol);
     void writeInheritanceGraph(OutputList &ol);
     void writeCollaborationGraph(OutputList &ol);
     void writeMemberGroups(OutputList &ol,bool showInline=FALSE);
@@ -383,10 +430,18 @@ class ClassDef : public Definition
     void writeMoreLink(OutputList &ol,const QCString &anchor);
     void writeDetailedDocumentationBody(OutputList &ol);
     
-    int countInheritedDecMembersRec(MemberList::ListType lt,ClassDef *inheritedFrom);
-    int countInheritedDecMembers(MemberList::ListType lt);
     int countAdditionalInheritedMembers();
     void writeAdditionalInheritedMembers(OutputList &ol);
+    void addClassAttributes(OutputList &ol);
+    int countMemberDeclarations(MemberListType lt,ClassDef *inheritedFrom,
+                                int lt2,bool invert,bool showAlways,QPtrDict<void> *visitedClasses);
+    int countInheritedDecMembers(MemberListType lt,
+                                 ClassDef *inheritedFrom,bool invert,bool showAlways,
+                                 QPtrDict<void> *visitedClasses);
+    void getTitleForMemberListType(MemberListType type,
+               QCString &title,QCString &subtitle);
+    QCString includeStatement() const;
+
     
     ClassDefImpl *m_impl;
 
@@ -483,14 +538,14 @@ class BaseClassList : public QList<BaseClassDef>
 {
   public:
    ~BaseClassList() {}
-    int compareItems(GCI item1,GCI item2)
+    int compareValues(const BaseClassDef *item1,const BaseClassDef *item2) const
     {
-      ClassDef *c1=((BaseClassDef *)item1)->classDef;
-      ClassDef *c2=((BaseClassDef *)item2)->classDef;
-      if (c1==0 || c2==0) 
+      const ClassDef *c1=item1->classDef;
+      const ClassDef *c2=item2->classDef;
+      if (c1==0 || c2==0)
         return FALSE;
       else
-        return stricmp(c1->name(),c2->name());
+        return qstricmp(c1->name(),c2->name());
     }
 };
 
