@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -35,7 +35,7 @@ Define::Define(const Define &d)
 { 
   //name=d.name; definition=d.definition; fileName=d.fileName; 
   lineNr=d.lineNr; 
-  lineNr=d.columnNr;
+  columnNr=d.columnNr;
   nargs=d.nargs; 
   undef=d.undef; 
   varArgs=d.varArgs;
