@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: definition.h,v 1.21 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,12 +18,10 @@
 #ifndef DEFINITION_H
 #define DEFINITION_H
 
-#include "qtbc.h"
 #include <qlist.h>
 #include <qdict.h>
-#include <sys/types.h>
 
-#include "lockingptr.h"
+#include "types.h"
 
 class FileDef;
 class OutputList;
@@ -36,16 +34,9 @@ struct ListItemInfo;
 struct SectionInfo;
 class Definition;
 class DefinitionImpl;
-
-#if 0
-struct ReachableDefinition
-{
-  ReachableDefinition(Definition *d,int dist) : def(d), distance(dist) {}
-  Definition *def;
-  int distance;
-};
-#endif
-
+class FTextStream;
+  
+/** Data associated with a detailed description. */
 struct DocInfo
 {
     QCString doc;  
@@ -53,6 +44,7 @@ struct DocInfo
     QCString file;
 };
 
+/** Data associated with a brief description. */
 struct BriefInfo
 {
     QCString doc;  
@@ -61,14 +53,15 @@ struct BriefInfo
     QCString file;
 };
 
+/** Data associated with description found in the body. */
 struct BodyInfo
 {
-    int      startLine;   // line number of the start of the definition
-    int      endLine;     // line number of the end of the definition
-    FileDef *fileDef;     // file definition containing the function body
+    int      startLine;   //!< line number of the start of the definition
+    int      endLine;     //!< line number of the end of the definition
+    FileDef *fileDef;     //!< file definition containing the function body
 };
     
-/*! Abstract interface for a Definition or DefinitionList */
+/** Abstract interface for a Definition or DefinitionList */
 class DefinitionIntf
 {
   public:
@@ -91,17 +84,18 @@ class DefinitionIntf
     virtual DefType definitionType() const = 0;
 };
 
-/*! The common base class of all entity definitions found in the sources. 
+/** The common base class of all entity definitions found in the sources. 
+ *
  *  This can be a class or a member function, or a file, or a namespace, etc.
  *  Use definitionType() to find which type of definition this is.
  */
-class Definition : public DefinitionIntf, public LockableObj
+class Definition : public DefinitionIntf
 {
   public:
     
     /*! Create a new definition */
     Definition(
-        const char *defFileName,int defLine,
+        const char *defFileName,int defLine,int defColumn,
         const char *name,const char *b=0,const char *d=0,
         bool isSymbol=TRUE);
 
@@ -115,6 +109,9 @@ class Definition : public DefinitionIntf, public LockableObj
     /*! Returns the name of the definition */
     const QCString& name() const { return m_name; }
 
+    /*! Returns the name of the definition as it appears in the output */
+    virtual QCString displayName(bool includeScope=TRUE) const = 0;
+
     /*! Returns the local name without any scope qualifiers. */
     QCString localName() const;
 
@@ -131,22 +128,28 @@ class Definition : public DefinitionIntf, public LockableObj
      */
     virtual QCString getOutputFileBase() const = 0;
 
-    /*! Returns the name of the source listing of this file. */
-    virtual QCString getSourceFileBase() const { ASSERT(0); return "NULL"; }
+    /*! Returns the anchor within a page where this item can be found */
+    virtual QCString anchor() const = 0;
+
+    /*! Returns the name of the source listing of this definition. */
+    virtual QCString getSourceFileBase() const;
+
+    /*! Returns the anchor of the source listing of this definition. */
+    virtual QCString getSourceAnchor() const;
 
     /*! Returns the detailed description of this definition */
-    QCString documentation() const;
-    
+    virtual QCString documentation() const;
+
     /*! Returns the line number at which the detailed documentation was found. */
     int docLine() const;
 
-    /*! Returns the file in which the detailed documentation block was found. 
+    /*! Returns the file in which the detailed documentation block was found.
      *  This can differ from getDefFileName().
      */
     QCString docFile() const;
 
     /*! Returns the brief description of this definition. This can include commands. */
-    QCString briefDescription() const;
+    virtual QCString briefDescription(bool abbreviate=FALSE) const;
 
     /*! Returns a plain text version of the brief description suitable for use
      *  as a tool tip. 
@@ -178,7 +181,10 @@ class Definition : public DefinitionIntf, public LockableObj
     QCString getDefFileExtension() const;
 
     /*! returns the line number at which the definition was found */
-    int getDefLine() const;
+    int getDefLine() const { return m_defLine; }
+
+    /*! returns the column number at which the definition was found */
+    int getDefColumn() const { return m_defColumn; }
 
     /*! Returns TRUE iff the definition is documented 
      *  (which could be generated documentation) 
@@ -238,24 +244,38 @@ class Definition : public DefinitionIntf, public LockableObj
     /*! Returns the file in which the body of this item is located or 0 if no
      *  body is available.
      */
-    FileDef *getBodyDef();
+    FileDef *getBodyDef() const;
 
-    LockingPtr<GroupList> partOfGroups() const;
+    /** Returns the programming language this definition was written in. */
+    SrcLangExt getLanguage() const;
 
-    LockingPtr< QList<ListItemInfo> > xrefListItems() const;
+    GroupList *partOfGroups() const;
+
+    QList<ListItemInfo> *xrefListItems() const;
 
     virtual Definition *findInnerCompound(const char *name);
     virtual Definition *getOuterScope() const;
 
-    LockingPtr<MemberSDict> getReferencesMembers() const;
-    LockingPtr<MemberSDict> getReferencedByMembers() const;
+    MemberSDict *getReferencesMembers() const;
+    MemberSDict *getReferencedByMembers() const;
+
+    bool hasSections() const;
+    bool hasSources() const;
+
+    /** returns TRUE if this class has a brief description */
+    bool hasBriefDescription() const;
+
+    QCString id() const;
 
     //-----------------------------------------------------------------------------------
     // ----  setters -----
     //-----------------------------------------------------------------------------------
 
     /*! Sets a new \a name for the definition */
-    void setName(const char *name);
+    virtual void setName(const char *name);
+
+    /*! Sets a unique id for the symbol. Used for libclang integration. */
+    void setId(const char *name);
 
     /*! Sets the documentation of this definition to \a d. */
     virtual void setDocumentation(const char *d,const char *docFile,int docLine,bool stripWhiteSpace=TRUE);
@@ -290,9 +310,10 @@ class Definition : public DefinitionIntf, public LockableObj
     virtual void addInnerCompound(Definition *d);
     virtual void setOuterScope(Definition *d);
 
-    void setHidden(bool b);
+    virtual void setHidden(bool b);
 
     void setArtificial(bool b);
+    void setLanguage(SrcLangExt lang);
 
     //-----------------------------------------------------------------------------------
     // --- actions ----
@@ -304,8 +325,9 @@ class Definition : public DefinitionIntf, public LockableObj
     void writeSourceRefs(OutputList &ol,const char *scopeName);
     void writeSourceReffedBy(OutputList &ol,const char *scopeName);
     void makePartOfGroup(GroupDef *gd);
-    void writePathFragment(OutputList &ol) const;
+    //void writePathFragment(OutputList &ol) const;
     void writeNavigationPath(OutputList &ol) const;
+    QCString navigationPathAsString() const;
     virtual void writeQuickMemberLinks(OutputList &,MemberDef *) const {}
     virtual void writeSummaryLinks(OutputList &) {}
     QCString pathFragment() const;
@@ -313,19 +335,17 @@ class Definition : public DefinitionIntf, public LockableObj
     /*! Writes the documentation anchors of the definition to 
      *  the Doxygen::tagFile stream.
      */
-    void writeDocAnchorsToTagFile();
-
-  protected:
+    void writeDocAnchorsToTagFile(FTextStream &);
     void setLocalName(const QCString name);
 
-    virtual void flushToDisk() const;
-    virtual void loadFromDisk() const;
-    virtual void makeResident() const;
-    void lock() const {}
-    void unlock() const {}
+    void addSectionsToIndex();
+    void writeToc(OutputList &ol);
+
+  protected:
+
+    Definition(const Definition &d);
 
   private: 
-
     static void addToMap(const char *name,Definition *d);
     static void removeFromMap(Definition *d);
 
@@ -337,28 +357,29 @@ class Definition : public DefinitionIntf, public LockableObj
     void _setBriefDescription(const char *b,const char *briefFile,int briefLine);
     void _setDocumentation(const char *d,const char *docFile,int docLine,bool stripWhiteSpace,bool atTop);
     void _setInbodyDocumentation(const char *d,const char *docFile,int docLine);
-    bool _docsAlreadyAdded(const QCString &doc);
+    bool _docsAlreadyAdded(const QCString &doc,QCString &sigList);
     DefinitionImpl *m_impl; // internal structure holding all private data
     QCString m_name;
     bool m_isSymbol;
     QCString m_symbolName;
-
+    int m_defLine;
+    int m_defColumn;
 };
 
+/** A list of Definition objects. */
 class DefinitionList : public QList<Definition>, public DefinitionIntf
 {
   public:
     ~DefinitionList() {}
     DefType definitionType() const { return TypeSymbolList; }
-    int compareItems(GCI item1,GCI item2)
+    int compareValues(const Definition *item1,const Definition *item2) const
     {
-      return stricmp(((Definition *)item1)->name(),
-                     ((Definition *)item2)->name()
-                    );
+      return qstricmp(item1->name(),item2->name());
     }
 
 };
 
+/** An iterator for Definition objects in a DefinitionList. */
 class DefinitionListIterator : public QListIterator<Definition>
 {
   public:
@@ -367,4 +388,12 @@ class DefinitionListIterator : public QListIterator<Definition>
     ~DefinitionListIterator() {}
 };
 
+/** Reads a fragment from file \a fileName starting with line \a startLine
+ *  and ending with line \a endLine. The result is returned as a string 
+ *  via \a result. The function returns TRUE if successful and FALSE 
+ *  in case of an error.
+ */
+bool readCodeFragment(const char *fileName, 
+                      int &startLine,int &endLine,
+                      QCString &result);
 #endif
