@@ -18,7 +18,7 @@
 
 static int g_dirCount=0;
 
-DirDef::DirDef(const char *path) : Definition(path,1,1,path)
+DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
 {
   bool fullPathNames = Config_getBool("FULL_PATH_NAMES");
   // get display name (stipping the paths mentioned in STRIP_FROM_PATH)
@@ -80,16 +80,15 @@ void DirDef::addFile(FileDef *fd)
 
 static QCString encodeDirName(const QCString &anchor)
 {
-  QCString result;
-
   // convert to md5 hash
   uchar md5_sig[16];
   QCString sigStr(33);
   MD5Buffer((const unsigned char *)anchor.data(),anchor.length(),md5_sig);
-  MD5SigToString(md5_sig,sigStr.data(),33);
+  MD5SigToString(md5_sig,sigStr.rawData(),33);
   return sigStr;
 
   // old algorithm
+//  QCString result;
 
 //  int l = anchor.length(),i;
 //  for (i=0;i<l;i++)
@@ -150,6 +149,7 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
         // ol.newParagraph();  // FIXME:PARA
         ol.enableAll();
         ol.disableAllBut(OutputGenerator::Man);
+        ol.enable(OutputGenerator::Latex);
         ol.writeString("\n\n");
       ol.popGeneratorState();
     }
@@ -224,8 +224,9 @@ void DirDef::writeSubDirList(OutputList &ol)
     ol.parseText(theTranslator->trDir(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    DirDef *dd=m_subdirs.first();
-    while (dd)
+    QListIterator<DirDef> it(m_subdirs);
+    DirDef *dd;
+    for (;(dd=it.current());++it)
     {
       ol.startMemberDeclaration();
       ol.startMemberItem(dd->getOutputFileBase(),0);
@@ -233,10 +234,6 @@ void DirDef::writeSubDirList(OutputList &ol)
       ol.insertMemberAlign();
       ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),0,dd->shortName());
       ol.endMemberItem();
-      if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-      {
-        Doxygen::tagFile << "    <dir>" << convertToXML(dd->displayName()) << "</dir>" << endl;
-      }
       if (!dd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
         ol.startMemberDescription(dd->getOutputFileBase());
@@ -250,7 +247,6 @@ void DirDef::writeSubDirList(OutputList &ol)
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(0,0);
-      dd=m_subdirs.next();
     }
 
     ol.endMemberList();
@@ -266,8 +262,9 @@ void DirDef::writeFileList(OutputList &ol)
     ol.parseText(theTranslator->trFile(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    FileDef *fd=m_fileList->first();
-    while (fd)
+    QListIterator<FileDef> it(*m_fileList);
+    FileDef *fd;
+    for (;(fd=it.current());++it)
     {
       ol.startMemberDeclaration();
       ol.startMemberItem(fd->getOutputFileBase(),0);
@@ -295,10 +292,6 @@ void DirDef::writeFileList(OutputList &ol)
         ol.endTextLink();
         ol.popGeneratorState();
       }
-      if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-      {
-        Doxygen::tagFile << "    <file>" << convertToXML(fd->name()) << "</file>" << endl;
-      }
       ol.endMemberItem();
       if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
@@ -313,7 +306,6 @@ void DirDef::writeFileList(OutputList &ol)
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(0,0);
-      fd=m_fileList->next();
     }
     ol.endMemberList();
   }
@@ -329,12 +321,69 @@ void DirDef::endMemberDeclarations(OutputList &ol)
   ol.endMemberSections();
 }
 
+QCString DirDef::shortTitle() const
+{
+  return theTranslator->trDirReference(m_shortName);
+}
+
+bool DirDef::hasDetailedDescription() const
+{
+  static bool repeatBrief = Config_getBool("REPEAT_BRIEF");
+  return (!briefDescription().isEmpty() && repeatBrief) || !documentation().isEmpty();
+}
+
+void DirDef::writeTagFile(FTextStream &tagFile)
+{
+  tagFile << "  <compound kind=\"dir\">" << endl;
+  tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
+  tagFile << "    <path>" << convertToXML(name()) << "</path>" << endl;
+  tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
+  QListIterator<LayoutDocEntry> eli(
+      LayoutDocManager::instance().docEntries(LayoutDocManager::Directory));
+  LayoutDocEntry *lde;
+  for (eli.toFirst();(lde=eli.current());++eli)
+  {
+    switch (lde->kind())
+    {
+      case LayoutDocEntry::DirSubDirs:
+        {
+          if (m_subdirs.count()>0)
+          {
+            DirDef *dd;
+            QListIterator<DirDef> it(m_subdirs);
+            for (;(dd=it.current());++it)
+            {
+              tagFile << "    <dir>" << convertToXML(dd->displayName()) << "</dir>" << endl;
+            }
+          }
+        }
+        break;
+      case LayoutDocEntry::DirFiles:
+        {
+          if (m_fileList->count()>0)
+          {
+            QListIterator<FileDef> it(*m_fileList);
+            FileDef *fd;
+            for (;(fd=it.current());++it)
+            {
+              tagFile << "    <file>" << convertToXML(fd->name()) << "</file>" << endl;
+            }
+          }
+        }
+        break;
+      default:
+        break;
+    }
+  }
+  writeDocAnchorsToTagFile(tagFile);
+  tagFile << "  </compound>" << endl;
+}
+
 void DirDef::writeDocumentation(OutputList &ol)
 {
   static bool generateTreeView = Config_getBool("GENERATE_TREEVIEW");
   ol.pushGeneratorState();
   
-  QCString shortTitle=theTranslator->trDirReference(m_shortName);
   QCString title=theTranslator->trDirReference(m_dispName);
   startFile(ol,getOutputFileBase(),name(),title,HLI_None,!generateTreeView);
 
@@ -348,7 +397,7 @@ void DirDef::writeDocumentation(OutputList &ol)
   startTitle(ol,getOutputFileBase());
   ol.pushGeneratorState();
     ol.disableAllBut(OutputGenerator::Html);
-    ol.parseText(shortTitle);
+    ol.parseText(shortTitle());
     ol.enableAll();
     ol.disable(OutputGenerator::Html);
     ol.parseText(title);
@@ -356,14 +405,6 @@ void DirDef::writeDocumentation(OutputList &ol)
   endTitle(ol,getOutputFileBase(),title);
   ol.startContents();
 
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    Doxygen::tagFile << "  <compound kind=\"dir\">" << endl;
-    Doxygen::tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
-    Doxygen::tagFile << "    <path>" << convertToXML(name()) << "</path>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
-  }
-  
   //---------------------------------------- start flexible part -------------------------------
 
   SrcLangExt lang = getLanguage();
@@ -439,12 +480,6 @@ void DirDef::writeDocumentation(OutputList &ol)
 
   //---------------------------------------- end flexible part -------------------------------
 
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    writeDocAnchorsToTagFile();
-    Doxygen::tagFile << "  </compound>" << endl;
-  }
-
   ol.endContents();
 
   endFileWithNavPath(this,ol);
@@ -580,10 +615,8 @@ bool DirDef::depGraphIsTrivial() const
 
 //----------------------------------------------------------------------
 
-int FilePairDict::compareItems(QCollection::Item item1,QCollection::Item item2)
+int FilePairDict::compareValues(const FilePair *left,const FilePair *right) const
 {
-  FilePair *left  = (FilePair*)item1;
-  FilePair *right = (FilePair*)item2;
   int orderHi = qstricmp(left->source()->name(),right->source()->name());
   int orderLo = qstricmp(left->destination()->name(),right->destination()->name());
   return orderHi==0 ? orderLo : orderHi;
