@@ -3,7 +3,7 @@
  * 
  *
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -32,6 +32,56 @@
 #include "config.h"
 #include "filedef.h"
 #include "msc.h"
+#include "dia.h"
+#include "htmlentity.h"
+#include "plantuml.h"
+
+static void visitPreStart(FTextStream &t, const bool hasCaption, QCString name,  QCString width,  QCString height)
+{
+  QCString tmpStr;
+  t << "    <figure>" << endl;
+  t << "        <title></title>" << endl;
+  t << "        <mediaobject>" << endl;
+  t << "            <imageobject>" << endl;
+  t << "                <imagedata";
+  if (!width.isEmpty())
+  {
+    t << " width=\"" << convertToXML(width) << "\"";
+  }
+  else
+  {
+    t << " width=\"50%\"";
+  }
+  if (!height.isEmpty())
+  {
+    t << " depth=\"" << convertToXML(tmpStr) << "\"";
+  }
+  t << " align=\"center\" valign=\"middle\" scalefit=\"1\" fileref=\"" << name << "\">";
+  t << "</imagedata>" << endl;
+  t << "            </imageobject>" << endl;
+  if (hasCaption)
+  {
+    t << "        <caption>" << endl;
+  }
+}
+
+static void visitPostEnd(FTextStream &t, const bool hasCaption)
+{
+  t << endl;
+  if (hasCaption)
+  {
+    t << "        </caption>" << endl;
+  }
+  t << "        </mediaobject>" << endl;
+  t << "    </figure>" << endl;
+}
+
+static void visitCaption(DocbookDocVisitor *parent, QList<DocNode> children)
+{
+  QListIterator<DocNode> cli(children);
+  DocNode *n;
+  for (cli.toFirst();(n=cli.current());++cli) n->accept(parent);
+}
 
 DocbookDocVisitor::DocbookDocVisitor(FTextStream &t,CodeOutputInterface &ci)
   : DocVisitor(DocVisitor_Docbook), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE)
@@ -72,109 +122,14 @@ void DocbookDocVisitor::visit(DocWhiteSpace *w)
 void DocbookDocVisitor::visit(DocSymbol *s)
 {
   if (m_hide) return;
-  switch(s->symbol())
+  const char *res = HtmlEntityMapper::instance()->docbook(s->symbol());
+  if (res)
   {
-    case DocSymbol::BSlash:  m_t << "\\"; break;
-    case DocSymbol::At:      m_t << "@"; break;
-    case DocSymbol::Less:    m_t << "&lt;"; break;
-    case DocSymbol::Greater: m_t << "&gt;"; break;
-    case DocSymbol::Amp:     m_t << "&amp;"; break;
-    case DocSymbol::Dollar:  m_t << "$"; break;
-    case DocSymbol::Hash:    m_t << "#"; break;
-    case DocSymbol::DoubleColon: m_t << "::"; break;
-    case DocSymbol::Percent: m_t << "%"; break;
-    case DocSymbol::Copy:    m_t << "&#x00A9;"; break;
-    case DocSymbol::Tm:      m_t << "&#8482;"; break;
-    case DocSymbol::Reg:     m_t << "&#174;"; break;
-    case DocSymbol::Apos:    m_t << "&apos;"; break;
-    case DocSymbol::Quot:    m_t << "&quot;"; break;
-    case DocSymbol::Lsquo:   m_t << "&#8216;"; break;
-    case DocSymbol::Rsquo:   m_t << "&#8217;"; break;
-    case DocSymbol::Ldquo:   m_t << "&#8220;"; break;
-    case DocSymbol::Rdquo:   m_t << "&#8221;"; break;
-    case DocSymbol::Ndash:   m_t << "&#8211;"; break;
-    case DocSymbol::Mdash:   m_t << "&#8212;"; break;
-    case DocSymbol::Uml:     m_t << "&#168;"; break;
-    case DocSymbol::Acute:   m_t << "&#180;"; break;
-    case DocSymbol::Grave:   m_t << "&#192;"; break;
-    case DocSymbol::Circ:    m_t << "&#710;"; break;
-    case DocSymbol::Tilde:   m_t << "&#732;"; break;
-    case DocSymbol::Szlig:   m_t << "&#223;"; break;
-    case DocSymbol::Cedil:   m_t << "&#184;"; break;
-    case DocSymbol::Ring:    m_t << "&#197;"; break;
-    case DocSymbol::Slash:   m_t << "&#216;"; break;
-    case DocSymbol::Nbsp:    m_t << "&#160;"; break;
-    case DocSymbol::Aelig:   m_t << "&#230;"; break;
-    case DocSymbol::AElig:   m_t << "&#198;"; break;
-    case DocSymbol::GrkGamma:      m_t << "&#915;"; break;
-    case DocSymbol::GrkDelta:      m_t << "&#916;"; break;
-    case DocSymbol::GrkTheta:      m_t << "&#920;"; break;
-    case DocSymbol::GrkLambda:     m_t << "&#923;"; break;
-    case DocSymbol::GrkXi:         m_t << "&#926;"; break;
-    case DocSymbol::GrkPi:         m_t << "&#928;"; break;
-    case DocSymbol::GrkSigma:      m_t << "&#931;"; break;
-    case DocSymbol::GrkUpsilon:    m_t << "&#933;"; break;
-    case DocSymbol::GrkPhi:        m_t << "&#934;"; break;
-    case DocSymbol::GrkPsi:        m_t << "&#936;"; break;
-    case DocSymbol::GrkOmega:      m_t << "&#937;"; break;
-    case DocSymbol::Grkalpha:      m_t << "&#945;"; break;
-    case DocSymbol::Grkbeta:       m_t << "&#946;"; break;
-    case DocSymbol::Grkgamma:      m_t << "&#947;"; break;
-    case DocSymbol::Grkdelta:      m_t << "&#948;"; break;
-    case DocSymbol::Grkepsilon:    m_t << "&#949;"; break;
-    case DocSymbol::Grkzeta:       m_t << "&#950;"; break;
-    case DocSymbol::Grketa:        m_t << "&#951;"; break;
-    case DocSymbol::Grktheta:      m_t << "&#952;"; break;
-    case DocSymbol::Grkiota:       m_t << "&#953;"; break;
-    case DocSymbol::Grkkappa:      m_t << "&#954;"; break;
-    case DocSymbol::Grklambda:     m_t << "&#955;"; break;
-    case DocSymbol::Grkmu:         m_t << "&#956;"; break;
-    case DocSymbol::Grknu:         m_t << "&#957;"; break;
-    case DocSymbol::Grkxi:         m_t << "&#958;"; break;
-    case DocSymbol::Grkpi:         m_t << "&#960;"; break;
-    case DocSymbol::Grkrho:        m_t << "&#961;"; break;
-    case DocSymbol::Grksigma:      m_t << "&#963;"; break;
-    case DocSymbol::Grktau:        m_t << "&#964;"; break;
-    case DocSymbol::Grkupsilon:    m_t << "&#965;"; break;
-    case DocSymbol::Grkphi:        m_t << "&#966;"; break;
-    case DocSymbol::Grkchi:        m_t << "&#967;"; break;
-    case DocSymbol::Grkpsi:        m_t << "&#968;"; break;
-    case DocSymbol::Grkomega:      m_t << "&#969;"; break;
-    case DocSymbol::Grkvarsigma:   m_t << "&#962;"; break;
-    case DocSymbol::Section:       m_t << "<simplesect/>"; break;
-    case DocSymbol::Degree:        m_t << "&#176;"; break;
-    case DocSymbol::Prime:         m_t << "&#8242;"; break;
-    case DocSymbol::DoublePrime:   m_t << "&#8243;"; break;
-    case DocSymbol::Infinity:      m_t << "&#8734;"; break;
-    case DocSymbol::EmptySet:      m_t << "&#8709;"; break;
-    case DocSymbol::PlusMinus:     m_t << "&#177;"; break;
-    case DocSymbol::Times:         m_t << "&#215;"; break;
-    case DocSymbol::Minus:         m_t << "&#8722;"; break;
-    case DocSymbol::CenterDot:     m_t << "&#8901;"; break;
-    case DocSymbol::Partial:       m_t << "&#8706;"; break;
-    case DocSymbol::Nabla:         m_t << "&#8711;"; break;
-    case DocSymbol::SquareRoot:    m_t << "&#8730;"; break;
-    case DocSymbol::Perpendicular: m_t << "&#8869;"; break;
-    case DocSymbol::Sum:           m_t << "&#8721;"; break;
-    case DocSymbol::Integral:      m_t << "&#8747;"; break;
-    case DocSymbol::Product:       m_t << "&#8719;"; break;
-    case DocSymbol::Similar:       m_t << "&#8764;"; break;
-    case DocSymbol::Approx:        m_t << "&#8776;"; break;
-    case DocSymbol::NotEqual:      m_t << "&#8800;"; break;
-    case DocSymbol::Equivalent:    m_t << "&#8801;"; break;
-    case DocSymbol::Proportional:  m_t << "&#8733;"; break;
-    case DocSymbol::LessEqual:     m_t << "&#8804;"; break;
-    case DocSymbol::GreaterEqual:  m_t << "&#8805;"; break;
-    case DocSymbol::LeftArrow:     m_t << "&#8592;"; break;
-    case DocSymbol::RightArrow:    m_t << "&#8594;"; break;
-    case DocSymbol::SetIn:         m_t << "&#8712;"; break;
-    case DocSymbol::SetNotIn:      m_t << "&#8713;"; break;
-    case DocSymbol::LeftCeil:      m_t << "&#8968;"; break;
-    case DocSymbol::RightCeil:     m_t << "&#8969;"; break;
-    case DocSymbol::LeftFloor:     m_t << "&#8970;"; break;
-    case DocSymbol::RightFloor:    m_t << "&#8971;"; break;
-    default:
-                                   err("unknown symbol found\n");
+    m_t << res;
+  }
+  else
+  {
+    err("DocBook: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance()->html(s->symbol(),TRUE));
   }
 }
 
@@ -253,16 +208,16 @@ void DocbookDocVisitor::visit(DocVerbatim *s)
   switch(s->type())
   {
     case DocVerbatim::Code: // fall though
-      m_t << "<programlisting>";
+      m_t << "<literallayout><computeroutput>";
       Doxygen::parserManager->getParser(m_langExt)
         ->parseCode(m_ci,s->context(),s->text(),langExt,
             s->isExample(),s->exampleFile());
-      m_t << "</programlisting>";
+      m_t << "</computeroutput></literallayout>";
       break;
     case DocVerbatim::Verbatim:
-      m_t << "<programlisting><literallayout>";
+      m_t << "<literallayout>";
       filter(s->text());
-      m_t << "</literallayout></programlisting>";
+      m_t << "</literallayout>";
       break;
     case DocVerbatim::HtmlOnly:    
       break;
@@ -282,8 +237,10 @@ void DocbookDocVisitor::visit(DocVerbatim *s)
       {
         static int dotindex = 1;
         QCString baseName(4096);
+        QCString name;
         QCString stext = s->text();
         m_t << "<para>" << endl;
+        name.sprintf("%s%d", "dot_inline_dotgraph_", dotindex);
         baseName.sprintf("%s%d",
             (Config_getString("DOCBOOK_OUTPUT")+"/inline_dotgraph_").data(),
             dotindex++
@@ -295,45 +252,50 @@ void DocbookDocVisitor::visit(DocVerbatim *s)
         }
         file.writeBlock( stext, stext.length() );
         file.close();
-        m_t << "    <figure>" << endl;
-        m_t << "        <title></title>" << endl;
-        m_t << "        <mediaobject>" << endl;
-        m_t << "            <imageobject>" << endl;
-        writeDotFile(baseName);
-        m_t << "            </imageobject>" << endl;
-        m_t << "       </mediaobject>" << endl;
-        m_t << "    </figure>" << endl;
+        writeDotFile(baseName, s);
         m_t << "</para>" << endl;
       }
       break;
     case DocVerbatim::Msc:
-      static int mscindex = 1;
-      QCString baseName(4096);
-      QCString stext = s->text();
-      m_t << "<para>" << endl;
-      baseName.sprintf("%s%d",
-          (Config_getString("DOCBOOK_OUTPUT")+"/inline_mscgraph_").data(),
-          mscindex++
-          );
-      QFile file(baseName+".msc");
-      if (!file.open(IO_WriteOnly))
       {
-        err("Could not open file %s.msc for writing\n",baseName.data());
+        static int mscindex = 1;
+        QCString baseName(4096);
+        QCString name;
+        QCString stext = s->text();
+        m_t << "<para>" << endl;
+        name.sprintf("%s%d", "msc_inline_mscgraph_", mscindex);
+        baseName.sprintf("%s%d",
+            (Config_getString("DOCBOOK_OUTPUT")+"/inline_mscgraph_").data(),
+            mscindex++
+            );
+        QFile file(baseName+".msc");
+        if (!file.open(IO_WriteOnly))
+        {
+          err("Could not open file %s.msc for writing\n",baseName.data());
+        }
+        QCString text = "msc {";
+        text+=stext;
+        text+="}";
+        file.writeBlock( text, text.length() );
+        file.close();
+        writeMscFile(baseName,s);
+        m_t << "</para>" << endl;
+      }
+      break;
+    case DocVerbatim::PlantUML:
+      {
+        static QCString docbookOutput = Config_getString("DOCBOOK_OUTPUT");
+        QCString baseName = writePlantUMLSource(docbookOutput,s->exampleFile(),s->text());
+        QCString shortName = baseName;
+        int i;
+        if ((i=shortName.findRev('/'))!=-1)
+        {
+          shortName=shortName.right(shortName.length()-i-1);
+        }
+        m_t << "<para>" << endl;
+        writePlantUMLFile(baseName,s);
+        m_t << "</para>" << endl;
       }
-      QCString text = "msc {";
-      text+=stext;
-      text+="}";
-      file.writeBlock( text, text.length() );
-      file.close();
-      m_t << "    <figure>" << endl;
-      m_t << "        <title></title>" << endl;
-      m_t << "        <mediaobject>" << endl;
-      m_t << "            <imageobject>" << endl;
-      writeMscFile(baseName);
-      m_t << "            </imageobject>" << endl;
-      m_t << "       </mediaobject>" << endl;
-      m_t << "    </figure>" << endl;
-      m_t << "</para>" << endl;
       break;
   }
 }
@@ -352,7 +314,7 @@ void DocbookDocVisitor::visit(DocInclude *inc)
   {
     case DocInclude::IncWithLines:
       {
-        m_t << "<programlisting>";
+        m_t << "<literallayout><computeroutput>";
         QFileInfo cfi( inc->file() );
         FileDef fd( cfi.dirPath().utf8(), cfi.fileName().utf8() );
         Doxygen::parserManager->getParser(inc->extension())
@@ -361,30 +323,32 @@ void DocbookDocVisitor::visit(DocInclude *inc)
               langExt,
               inc->isExample(),
               inc->exampleFile(), &fd);
-        m_t << "</programlisting>";
+        m_t << "</computeroutput></literallayout>";
       }
       break;
     case DocInclude::Include:
-      m_t << "<programlisting>";
+      m_t << "<literallayout><computeroutput>";
       Doxygen::parserManager->getParser(inc->extension())
         ->parseCode(m_ci,inc->context(),
             inc->text(),
             langExt,
             inc->isExample(),
             inc->exampleFile());
-      m_t << "</programlisting>";
+      m_t << "</computeroutput></literallayout>";
       break;
     case DocInclude::DontInclude:
       break;
     case DocInclude::HtmlInclude:
       break;
+    case DocInclude::LatexInclude:
+      break;
     case DocInclude::VerbInclude:
       m_t << "<verbatim>";
       filter(inc->text());
       m_t << "</verbatim>";
       break;
     case DocInclude::Snippet:
-      m_t << "<programlisting>";
+      m_t << "<literallayout><computeroutput>";
       Doxygen::parserManager->getParser(inc->extension())
         ->parseCode(m_ci,
             inc->context(),
@@ -393,7 +357,7 @@ void DocbookDocVisitor::visit(DocInclude *inc)
             inc->isExample(),
             inc->exampleFile()
             );
-      m_t << "</programlisting>";
+      m_t << "</computeroutput></literallayout>";
       break;
   }
 }
@@ -909,14 +873,19 @@ void DocbookDocVisitor::visitPost(DocHtmlHeader *)
 
 void DocbookDocVisitor::visitPre(DocImage *img)
 {
-  if (img->type()==DocImage::Latex)
+  if (img->type()==DocImage::DocBook)
   {
     if (m_hide) return;
     m_t << endl;
-    m_t << "    <figure>" << endl;
-    m_t << "        <title>";
-  } 
-  else 
+    QCString baseName=img->name();
+    int i;
+    if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
+    {
+      baseName=baseName.right(baseName.length()-i-1);
+    }
+    visitPreStart(m_t, img -> hasCaption(), baseName, img -> width(), img -> height());
+  }
+  else
   {
     pushEnabled();
     m_hide=TRUE;
@@ -925,38 +894,17 @@ void DocbookDocVisitor::visitPre(DocImage *img)
 
 void DocbookDocVisitor::visitPost(DocImage *img)
 {
-  if (img->type()==DocImage::Latex)
+  if (img->type()==DocImage::DocBook)
   {
     if (m_hide) return;
-    QCString typevar;
-    m_t << "</title>" << endl;
-    m_t << "    <mediaobject>" << endl;
-    m_t << "        <imageobject>" << endl;
+    visitPostEnd(m_t, img -> hasCaption());
+    // copy the image to the output dir
     QCString baseName=img->name();
     int i;
     if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
     {
       baseName=baseName.right(baseName.length()-i-1);
     }
-    m_t << "            <imagedata";
-    if (!img->width().isEmpty())
-    {
-      m_t << " width=\"";
-      filter(img->width());
-      m_t << "\"";
-    }
-    else if (!img->height().isEmpty())
-    {
-      m_t << " depth=\"";
-      filter(img->height());
-      m_t << "\"";
-    }
-    m_t << " align=\"center\" fileref=\"" << baseName << "\">";
-    m_t << "</imagedata>" << endl;
-    m_t << "        </imageobject>" << endl;
-    m_t << "    </mediaobject>" << endl;
-    m_t << "    </figure>" << endl;
-    // copy the image to the output dir
     QCString m_file;
     bool ambig;
     FileDef *fd=findFileDef(Doxygen::imageNameDict, baseName, ambig);
@@ -1007,6 +955,18 @@ void DocbookDocVisitor::visitPost(DocMscFile *df)
   if (m_hide) return;
   endMscFile(df->hasCaption());
 }
+void DocbookDocVisitor::visitPre(DocDiaFile *df)
+{
+  if (m_hide) return;
+  startDiaFile(df->file(),df->width(),df->height(),df->hasCaption());
+}
+
+void DocbookDocVisitor::visitPost(DocDiaFile *df)
+{
+  if (m_hide) return;
+  endDiaFile(df->hasCaption());
+}
+
 void DocbookDocVisitor::visitPre(DocLink *lnk)
 {
   if (m_hide) return;
@@ -1059,37 +1019,25 @@ void DocbookDocVisitor::visitPost(DocSecRefList *)
 void DocbookDocVisitor::visitPre(DocParamSect *s)
 {
   if (m_hide) return;
+  m_t <<  endl;
+  m_t << "                <formalpara>" << endl;
+  m_t << "                    <title/>" << endl;
+  m_t << "                    <table frame=\"all\">" << endl;
+  m_t << "                        <title>";
   switch(s->type())
   {
-    case DocParamSect::Param:
-      {
-        m_t <<  endl;
-        m_t << "                <formalpara>" << endl;
-        m_t << "                    <title/>" << endl;
-        m_t << "                    <table frame=\"all\">" << endl;
-        m_t << "                        <title>Parameters</title>" << endl;
-        m_t << "                        <tgroup cols=\"2\" align=\"left\" colsep=\"1\" rowsep=\"1\">" << endl;
-        m_t << "                        <tbody>" << endl;
-        break;
-      }
-    case DocParamSect::RetVal:
-      {
-        m_t <<  endl;
-        m_t << "                <formalpara>" << endl;
-        m_t << "                    <title/>" << endl;
-        m_t << "                    <table frame=\"all\">" << endl;
-        m_t << "                        <title>Parameters</title>" << endl;
-        m_t << "                        <tgroup cols=\"2\" align=\"left\" colsep=\"1\" rowsep=\"1\">" << endl;
-        m_t << "                        <tbody>" << endl;
-        break;
-      }
-    case DocParamSect::Exception:
-      m_t << "exception"; break;
-    case DocParamSect::TemplateParam:
-      m_t << "templateparam"; break;
+    case DocParamSect::Param:         m_t << theTranslator->trParameters();         break;
+    case DocParamSect::RetVal:        m_t << theTranslator->trReturnValues();       break;
+    case DocParamSect::Exception:     m_t << theTranslator->trExceptions();         break;
+    case DocParamSect::TemplateParam: m_t << theTranslator->trTemplateParameters(); break;
     default:
       ASSERT(0);
   }
+  m_t << "                        </title>" << endl;
+  m_t << "                        <tgroup cols=\"2\" align=\"left\" colsep=\"1\" rowsep=\"1\">" << endl;
+  m_t << "                        <colspec colwidth=\"1*\"/>" << endl;
+  m_t << "                        <colspec colwidth=\"4*\"/>" << endl;
+  m_t << "                        <tbody>" << endl;
 }
 
 void DocbookDocVisitor::visitPost(DocParamSect *)
@@ -1108,17 +1056,17 @@ void DocbookDocVisitor::visitPre(DocParamList *pl)
   QListIterator<DocNode> li(pl->parameters());
   DocNode *param;
   m_t << "                            <row>" << endl;
-  if (!li.toFirst()) 
+  if (!li.toFirst())
   {
     m_t << "                                <entry></entry>" << endl;
-  } 
-  else 
+  }
+  else
   {
     m_t << "                                <entry>";
     int cnt = 0;
     for (li.toFirst();(param=li.current());++li)
     {
-      if (cnt) 
+      if (cnt)
       {
         m_t << ", ";
       }
@@ -1147,6 +1095,7 @@ void DocbookDocVisitor::visitPost(DocParamList *)
 void DocbookDocVisitor::visitPre(DocXRefItem *x)
 {
   if (m_hide) return;
+  if (x->title().isEmpty()) return;
   m_t << "<para><link linkend=\"";
   m_t << x->file() << "_1" << x->anchor();
   m_t << "\">";
@@ -1155,9 +1104,10 @@ void DocbookDocVisitor::visitPre(DocXRefItem *x)
   m_t << " ";
 }
 
-void DocbookDocVisitor::visitPost(DocXRefItem *)
+void DocbookDocVisitor::visitPost(DocXRefItem *x)
 {
   if (m_hide) return;
+  if (x->title().isEmpty()) return;
   m_t << "</para>";
 }
 
@@ -1223,6 +1173,15 @@ void DocbookDocVisitor::visitPost(DocVhdlFlow *)
   // TODO: to be implemented
 }
 
+void DocbookDocVisitor::visitPre(DocParBlock *)
+{
+}
+
+void DocbookDocVisitor::visitPost(DocParBlock *)
+{
+}
+
+
 void DocbookDocVisitor::filter(const char *str)
 {
   m_t << convertToXML(str);
@@ -1253,7 +1212,7 @@ void DocbookDocVisitor::popEnabled()
   delete v;
 }
 
-void DocbookDocVisitor::writeMscFile(const QCString &baseName)
+void DocbookDocVisitor::writeMscFile(const QCString &baseName, DocVerbatim *s)
 {
   QCString shortName = baseName;
   int i;
@@ -1263,9 +1222,24 @@ void DocbookDocVisitor::writeMscFile(const QCString &baseName)
   }
   QCString outDir = Config_getString("DOCBOOK_OUTPUT");
   writeMscGraphFromFile(baseName+".msc",outDir,shortName,MSC_BITMAP);
-  m_t << "                <imagedata";
-  m_t << " align=\"center\" fileref=\"" << shortName << ".png" << "\">";
-  m_t << "</imagedata>" << endl;
+  visitPreStart(m_t, s->hasCaption(), shortName, s->width(),s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
+}
+
+void DocbookDocVisitor::writePlantUMLFile(const QCString &baseName, DocVerbatim *s)
+{
+  QCString shortName = baseName;
+  int i;
+  if ((i=shortName.findRev('/'))!=-1)
+  {
+    shortName=shortName.right(shortName.length()-i-1);
+  }
+  QCString outDir = Config_getString("DOCBOOK_OUTPUT");
+  generatePlantUMLOutput(baseName,outDir,PUML_BITMAP);
+  visitPreStart(m_t, s->hasCaption(), shortName, s->width(),s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
 }
 
 void DocbookDocVisitor::startMscFile(const QCString &fileName,
@@ -1288,46 +1262,17 @@ void DocbookDocVisitor::startMscFile(const QCString &fileName,
   QCString outDir = Config_getString("DOCBOOK_OUTPUT");
   writeMscGraphFromFile(fileName,outDir,baseName,MSC_BITMAP);
   m_t << "<para>" << endl;
-  m_t << "    <figure>" << endl;
-  m_t << "        <title></title>" << endl;
-  m_t << "        <mediaobject>" << endl;
-  m_t << "            <imageobject>" << endl;
-  m_t << "                <imagedata";
-  if (!width.isEmpty())
-  {
-    m_t << " width=\"";
-    m_t << width;
-    m_t << "\"";
-  }
-  else if (!height.isEmpty())
-  {
-    m_t << " depth=\"";
-    m_t << height;
-    m_t << "\"";
-  }
-  m_t << " align=\"center\" fileref=\"" << baseName << ".png" << "\">";
-  m_t << "</imagedata>" << endl;
-  m_t << "            </imageobject>" << endl;
-  if (hasCaption)
-  {
-    m_t << "        <caption>" << endl;
-  }
+  visitPreStart(m_t, hasCaption, baseName + ".png",  width,  height);
 }
 
 void DocbookDocVisitor::endMscFile(bool hasCaption)
 {
   if (m_hide) return;
-  m_t << "endl";
-  if (hasCaption)
-  {
-    m_t << "        </caption>" << endl;
-  }
-  m_t << "        </mediaobject>" << endl;
-  m_t << "    </figure>" << endl;
+  visitPostEnd(m_t, hasCaption);
   m_t << "</para>" << endl;
 }
 
-void DocbookDocVisitor::writeDotFile(const QCString &baseName)
+void DocbookDocVisitor::writeDiaFile(const QCString &baseName, DocVerbatim *s)
 {
   QCString shortName = baseName;
   int i;
@@ -1336,10 +1281,55 @@ void DocbookDocVisitor::writeDotFile(const QCString &baseName)
     shortName=shortName.right(shortName.length()-i-1);
   }
   QCString outDir = Config_getString("DOCBOOK_OUTPUT");
-  writeDotGraphFromFile(baseName+".dot",outDir,shortName,BITMAP);
-  m_t << "                <imagedata";
-  m_t << " align=\"center\" fileref=\"" << shortName << ".png" << "\">";
-  m_t << "</imagedata>" << endl;
+  writeDiaGraphFromFile(baseName+".dia",outDir,shortName,DIA_BITMAP);
+  visitPreStart(m_t, s->hasCaption(), shortName, s->width(),s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
+}
+
+void DocbookDocVisitor::startDiaFile(const QCString &fileName,
+    const QCString &width,
+    const QCString &height,
+    bool hasCaption
+    )
+{
+  QCString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  }
+  if ((i=baseName.find('.'))!=-1)
+  {
+    baseName=baseName.left(i);
+  }
+  baseName.prepend("dia_");
+  QCString outDir = Config_getString("DOCBOOK_OUTPUT");
+  writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP);
+  m_t << "<para>" << endl;
+  visitPreStart(m_t, hasCaption, baseName + ".png",  width,  height);
+}
+
+void DocbookDocVisitor::endDiaFile(bool hasCaption)
+{
+  if (m_hide) return;
+  visitPostEnd(m_t, hasCaption);
+  m_t << "</para>" << endl;
+}
+
+void DocbookDocVisitor::writeDotFile(const QCString &baseName, DocVerbatim *s)
+{
+  QCString shortName = baseName;
+  int i;
+  if ((i=shortName.findRev('/'))!=-1)
+  {
+    shortName=shortName.right(shortName.length()-i-1);
+  }
+  QCString outDir = Config_getString("DOCBOOK_OUTPUT");
+  writeDotGraphFromFile(baseName+".dot",outDir,shortName,GOF_BITMAP);
+  visitPreStart(m_t, s->hasCaption(), baseName + ".dot", s->width(),s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
 }
 
 void DocbookDocVisitor::startDotFile(const QCString &fileName,
@@ -1360,44 +1350,17 @@ void DocbookDocVisitor::startDotFile(const QCString &fileName,
   }
   baseName.prepend("dot_");
   QCString outDir = Config_getString("DOCBOOK_OUTPUT");
-  writeDotGraphFromFile(fileName,outDir,baseName,BITMAP);
+  QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
+  writeDotGraphFromFile(fileName,outDir,baseName,GOF_BITMAP);
   m_t << "<para>" << endl;
-  m_t << "    <figure>" << endl;
-  m_t << "        <title></title>" << endl;
-  m_t << "        <mediaobject>" << endl;
-  m_t << "            <imageobject>" << endl;
-  m_t << "                <imagedata";
-  if (!width.isEmpty())
-  {
-    m_t << " width=\"";
-    m_t << width;
-    m_t << "\"";
-  }
-  else if (!height.isEmpty())
-  {
-    m_t << " depth=\"";
-    m_t << height;
-    m_t << "\"";
-  }
-  m_t << " align=\"center\" fileref=\"" << baseName << ".png" << "\">";
-  m_t << "</imagedata>" << endl;
-  m_t << "            </imageobject>" << endl;
-  if (hasCaption)
-  {
-    m_t << "        <caption>" << endl;
-  }
+  visitPreStart(m_t, hasCaption, baseName + "." + imgExt,  width,  height);
 }
 
 void DocbookDocVisitor::endDotFile(bool hasCaption)
 {
   if (m_hide) return;
-  m_t << "endl";
-  if (hasCaption)
-  {
-    m_t << "        </caption>" << endl;
-  }
-  m_t << "        </mediaobject>" << endl;
-  m_t << "    </figure>" << endl;
+  m_t << endl;
+  visitPostEnd(m_t, hasCaption);
   m_t << "</para>" << endl;
 }
 
