@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -63,7 +63,8 @@ enum Tokens
   RetVal_CloseXml       = 0x10013,
   RetVal_EndBlockQuote  = 0x10014,
   RetVal_CopyDoc        = 0x10015,
-  RetVal_EndInternal    = 0x10016
+  RetVal_EndInternal    = 0x10016,
+  RetVal_EndParBlock    = 0x10017
 };
 
 /** @brief Data associated with a token used by the comment block parser. */
@@ -159,5 +160,8 @@ void doctokenizerYYpushBackHtmlTag(const char *tag);
 void doctokenizerYYsetStateSnippet();
 void doctokenizerYYstartAutoList();
 void doctokenizerYYendAutoList();
+void doctokenizerYYsetStatePlantUML();
+void doctokenizerYYsetStateSetScope();
+void doctokenizerYYsetStatePlantUMLOpt();
 
 #endif
