@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: entry.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,6 +22,8 @@
 #include "util.h"
 #include "section.h"
 #include "doxygen.h"
+#include "filestorage.h"
+#include "arguments.h"
 
 //------------------------------------------------------------------
 
@@ -29,23 +31,6 @@
 
 //------------------------------------------------------------------
 
-/*! the argument list is documented if one of its
- *  arguments is documented 
- */
-bool ArgumentList::hasDocumentation() const
-{
-  bool hasDocs=FALSE;
-  ArgumentListIterator ali(*this);
-  Argument *a;
-  for (ali.toFirst();!hasDocs && (a=ali.current());++ali)
-  {
-    hasDocs = a->hasDocumentation(); 
-  }
-  return hasDocs;
-}
-
-//------------------------------------------------------------------
-
 int Entry::num=0;
 
 Entry::Entry()
@@ -96,8 +81,7 @@ Entry::Entry(const Entry &e)
   virt        = e.virt;
   args        = e.args;
   bitfields   = e.bitfields;
-  argList     = new ArgumentList;
-  argList->setAutoDelete(TRUE);
+  argList     = e.argList->deepCopy();
   tArgLists = 0;
   program     = e.program;
   initializer = e.initializer;
@@ -129,6 +113,7 @@ Entry::Entry(const Entry &e)
   anchors     = new QList<SectionInfo>;
   fileName    = e.fileName;
   startLine   = e.startLine;
+  startColumn = e.startColumn;
   if (e.sli)
   {
     sli = new QList<ListItemInfo>;
@@ -148,6 +133,7 @@ Entry::Entry(const Entry &e)
   hidden      = e.hidden;
   artificial  = e.artificial;
   groupDocType = e.groupDocType;
+  id          = e.id;
 
   m_parent    = e.m_parent;
   m_sublist   = new QList<Entry>;
@@ -184,27 +170,10 @@ Entry::Entry(const Entry &e)
     anchors->append(new SectionInfo(*s));
   }
 
-  // deep copy argument list
-  QListIterator<Argument> ali(*e.argList);
-  Argument *a;
-  for (;(a=ali.current());++ali)
-  {
-    argList->append(new Argument(*a));
-  }
-  argList->constSpecifier    = e.argList->constSpecifier;
-  argList->volatileSpecifier = e.argList->volatileSpecifier;
-  argList->pureSpecifier     = e.argList->pureSpecifier;
-  
-  // deep copy type contraint list
+  // deep copy type constraint list
   if (e.typeConstr)
   {
-    typeConstr  = new ArgumentList;
-    typeConstr->setAutoDelete(TRUE);
-    QListIterator<Argument> tcli(*e.typeConstr);
-    for (;(a=tcli.current());++tcli)
-    {
-      typeConstr->append(new Argument(*a));
-    }
+    typeConstr  = e.typeConstr->deepCopy();
   }
 
   // deep copy template argument lists
@@ -272,6 +241,7 @@ void Entry::reset()
   initializer.resize(0);
   initLines = -1;
   startLine = 1;
+  startColumn = 1;
   bodyLine = -1;
   endBodyLine = -1;
   mGrpId = -1;
@@ -290,6 +260,7 @@ void Entry::reset()
   subGrouping = TRUE;
   protection = Public;
   groupDocType = GROUPDOC_NORMAL;
+  id.resize(0);
   m_sublist->clear();
   extends->clear();
   groups->clear();
@@ -330,21 +301,7 @@ void Entry::createSubtreeIndex(EntryNav *nav,FileStorage *storage,FileDef *fd)
 
 void Entry::createNavigationIndex(EntryNav *rootNav,FileStorage *storage,FileDef *fd)
 {
-  //printf("createNavigationIndex(%p) sublist=%p\n",this,m_sublist);
-  if (m_sublist)
-  {
-    //printf("saveEntries: %d children\n",root->sublist->count());
-    // store all child entries of root, but keep the navigation info (=index)
-    QListIterator<Entry> eli(*m_sublist);
-    Entry *e;
-    for (eli.toFirst();(e=eli.current());++eli)
-    {
-      createSubtreeIndex(rootNav,storage,fd);
-    }
-    // remove all entries from root
-    //m_sublist->setAutoDelete(FALSE);
-    m_sublist->clear();
-  }
+  createSubtreeIndex(rootNav,storage,fd);
 }
 
 void Entry::addSpecialListItem(const char *listName,int itemId)
@@ -360,6 +317,12 @@ void Entry::addSpecialListItem(const char *listName,int itemId)
   sli->append(ili);
 }
 
+Entry *Entry::removeSubEntry(Entry *e)
+{
+ int i = m_sublist->find(e);
+ return i!=-1 ? m_sublist->take(i) : 0;
+}
+
 //------------------------------------------------------------------
 
 
