@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,8 +15,10 @@
  *
  */
 
-#ifndef CODE_H
-#define CODE_H
+#ifndef FORTRANCODE_H
+#define FORTRANCODE_H
+
+#include "types.h"
 
 class CodeOutputInterface;
 class FileDef;
@@ -27,7 +29,8 @@ class Definition;
 void parseFortranCode(CodeOutputInterface &,const char *,const QCString &, 
             bool ,const char *,FileDef *fd,
             int startLine,int endLine,bool inlineFragment,
-            MemberDef *memberDef,bool showLineNumbers,Definition *searchCtx);
+            MemberDef *memberDef,bool showLineNumbers,Definition *searchCtx,
+            bool collectRefs, FortranFormat format);
 void resetFortranCodeParserState();
 void codeFreeScanner();
 
