@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * 
  *
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,8 +28,13 @@
 #include "htmlgen.h"
 #include "parserintf.h"
 #include "msc.h"
+#include "dia.h"
 #include "util.h"
-
+#include "vhdldocgen.h"
+#include "filedef.h"
+#include "memberdef.h"
+#include "htmlentity.h"
+#include "plantuml.h"
 
 static const int NUM_HTML_LIST_TYPES = 4;
 static const char types[][NUM_HTML_LIST_TYPES] = {"1", "a", "i", "A"};
@@ -37,26 +42,33 @@ static const char types[][NUM_HTML_LIST_TYPES] = {"1", "a", "i", "A"};
 static QCString convertIndexWordToAnchor(const QString &word)
 {
   static char hex[] = "0123456789abcdef";
-  uint i;
   QCString result;
-  for (i=0;i<word.length();i++)
+  const char *str = word.data();
+  unsigned char c;
+  if (str)
   {
-    int c = word.at(i);
-    if (isId(c))
+    while ((c = *str++))
     {
-      result+=c;
-    }
-    else if (isspace(c))
-    {
-      result+="_";
-    }
-    else
-    {
-      char cs[3];
-      cs[0]=hex[c>>4];
-      cs[1]=hex[c&0xf];
-      cs[2]=0;
-      result+=cs;
+      if ((c >= 'a' && c <= 'z') || // ALPHA
+          (c >= 'A' && c <= 'A') || // ALPHA
+          (c >= '0' && c <= '9') || // DIGIT
+          c == '-' ||
+          c == '.' ||
+          c == '_' ||
+          c == '~'
+         )
+      {
+        result += c;
+      }
+      else
+      {
+        char enc[4];
+        enc[0] = '%';
+        enc[1] = hex[(c & 0xf0) >> 4];
+        enc[2] = hex[c & 0xf];
+        enc[3] = 0;
+        result += enc;
+      }
     }
   }
   return result;
@@ -84,7 +96,6 @@ static bool mustBeOutsideParagraph(DocNode *n)
         case DocNode::Kind_Internal:
           /* <div> */
         case DocNode::Kind_Include:
-        case DocNode::Kind_Verbatim:
         case DocNode::Kind_Image:
         case DocNode::Kind_SecRefList:
           /* <hr> */
@@ -96,7 +107,14 @@ static bool mustBeOutsideParagraph(DocNode *n)
         case DocNode::Kind_Copy:
           /* <blockquote> */
         case DocNode::Kind_HtmlBlockQuote:
+          /* \parblock */
+        case DocNode::Kind_ParBlock:
           return TRUE;
+        case DocNode::Kind_Verbatim:
+          {
+            DocVerbatim *dv = (DocVerbatim*)n;
+            return dv->type()!=DocVerbatim::HtmlOnly || dv->isBlock();
+          }
         case DocNode::Kind_StyleChange:
           return ((DocStyleChange*)n)->style()==DocStyleChange::Preformatted ||
                  ((DocStyleChange*)n)->style()==DocStyleChange::Div ||
@@ -130,10 +148,11 @@ static QString htmlAttribsToString(const HtmlAttribList &attribs)
 //-------------------------------------------------------------------------
 
 HtmlDocVisitor::HtmlDocVisitor(FTextStream &t,CodeOutputInterface &ci,
-                               const char *langExt) 
+                               Definition *ctx) 
   : DocVisitor(DocVisitor_Html), m_t(t), m_ci(ci), m_insidePre(FALSE), 
-                                 m_hide(FALSE), m_langExt(langExt)
+                                 m_hide(FALSE), m_ctx(ctx)
 {
+  if (ctx) m_langExt=ctx->getDefFileExtension();
 }
 
   //--------------------------------------
@@ -150,6 +169,7 @@ void HtmlDocVisitor::visit(DocWord *w)
 void HtmlDocVisitor::visit(DocLinkedWord *w)
 {
   if (m_hide) return;
+  //printf("linked word: %s\n",w->word().data());
   startLink(w->ref(),w->file(),w->relPath(),w->anchor(),w->tooltip());
   filter(w->word());
   endLink();
@@ -171,42 +191,14 @@ void HtmlDocVisitor::visit(DocWhiteSpace *w)
 void HtmlDocVisitor::visit(DocSymbol *s)
 {
   if (m_hide) return;
-  switch(s->symbol())
+  const char *res = HtmlEntityMapper::instance()->html(s->symbol());
+  if (res)
   {
-    case DocSymbol::BSlash:  m_t << "\\"; break;
-    case DocSymbol::At:      m_t << "@"; break;
-    case DocSymbol::Less:    m_t << "&lt;"; break;
-    case DocSymbol::Greater: m_t << "&gt;"; break;
-    case DocSymbol::Amp:     m_t << "&amp;"; break;
-    case DocSymbol::Dollar:  m_t << "$"; break;
-    case DocSymbol::Hash:    m_t << "#"; break;
-    case DocSymbol::DoubleColon: m_t << "::"; break;
-    case DocSymbol::Percent: m_t << "%"; break;
-    case DocSymbol::Copy:    m_t << "&copy;"; break;
-    case DocSymbol::Tm:      m_t << "&trade;"; break;
-    case DocSymbol::Reg:     m_t << "&reg;"; break;
-    case DocSymbol::Apos:    m_t << "'"; break;
-    case DocSymbol::Quot:    m_t << "\""; break;
-    case DocSymbol::Lsquo:   m_t << "&lsquo;"; break;
-    case DocSymbol::Rsquo:   m_t << "&rsquo;"; break;
-    case DocSymbol::Ldquo:   m_t << "&ldquo;"; break;
-    case DocSymbol::Rdquo:   m_t << "&rdquo;"; break;
-    case DocSymbol::Ndash:   m_t << "&ndash;"; break;
-    case DocSymbol::Mdash:   m_t << "&mdash;"; break;
-    case DocSymbol::Uml:     m_t << "&" << s->letter() << "uml;"; break;
-    case DocSymbol::Acute:   m_t << "&" << s->letter() << "acute;"; break;
-    case DocSymbol::Grave:   m_t << "&" << s->letter() << "grave;"; break;
-    case DocSymbol::Circ:    m_t << "&" << s->letter() << "circ;"; break;
-    case DocSymbol::Slash:   m_t << "&" << s->letter() << "slash;"; break;
-    case DocSymbol::Tilde:   m_t << "&" << s->letter() << "tilde;"; break;
-    case DocSymbol::Szlig:   m_t << "&szlig;"; break;
-    case DocSymbol::Cedil:   m_t << "&" << s->letter() << "cedil;"; break;
-    case DocSymbol::Ring:    m_t << "&" << s->letter() << "ring;"; break;
-    case DocSymbol::Nbsp:    m_t << "&#160;"; break;
-    case DocSymbol::AElig:   m_t << "&AElig;"; break;
-    case DocSymbol::Aelig:   m_t << "&aelig;"; break;
-    default:
-                             err("error: unknown symbol found\n");
+    m_t << res;
+  }
+  else
+  {
+    err("HTML: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance()->html(s->symbol(),TRUE));
   }
 }
 
@@ -253,13 +245,15 @@ void HtmlDocVisitor::visit(DocURL *u)
 void HtmlDocVisitor::visit(DocLineBreak *)
 {
   if (m_hide) return;
-  m_t << "<br/>\n";
+  m_t << "<br />\n";
 }
 
-void HtmlDocVisitor::visit(DocHorRuler *)
+void HtmlDocVisitor::visit(DocHorRuler *hr)
 {
   if (m_hide) return;
+  forceEndParagraph(hr);
   m_t << "<hr/>\n";
+  forceStartParagraph(hr);
 }
 
 void HtmlDocVisitor::visit(DocStyleChange *s)
@@ -331,6 +325,32 @@ void HtmlDocVisitor::visit(DocStyleChange *s)
 }
 
 
+static void visitPreCaption(FTextStream &t, DocVerbatim *s)
+{
+  if (s->hasCaption())
+  { 
+    t << "<div class=\"caption\">" << endl;
+  }
+}
+
+
+static void visitPostCaption(FTextStream &t, DocVerbatim *s)
+{
+  if (s->hasCaption())
+  {
+    t << "</div>" << endl;
+  }
+}
+
+
+static void visitCaption(HtmlDocVisitor *parent, QList<DocNode> children)
+{
+  QListIterator<DocNode> cli(children);
+  DocNode *n;
+  for (cli.toFirst();(n=cli.current());++cli) n->accept(parent);
+}
+
+
 void HtmlDocVisitor::visit(DocVerbatim *s)
 {
   if (m_hide) return;
@@ -339,14 +359,27 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
   {
     lang = s->language();
   }
+  SrcLangExt langExt = getLanguageFromFileName(lang);
   switch(s->type())
   {
     case DocVerbatim::Code: 
       forceEndParagraph(s);
       m_t << PREFRAG_START;
       Doxygen::parserManager->getParser(lang)
-                            ->parseCode(m_ci,s->context(),s->text(),
-                                        s->isExample(),s->exampleFile());
+                            ->parseCode(m_ci,
+                                        s->context(),
+                                        s->text(),
+                                        langExt,
+                                        s->isExample(),
+                                        s->exampleFile(),
+                                        0,     // fileDef
+                                        -1,    // startLine
+                                        -1,    // endLine
+                                        FALSE, // inlineFragment
+                                        0,     // memberDef
+                                        TRUE,  // show line numbers
+                                        m_ctx  // search context
+                                       );
       m_t << PREFRAG_END;
       forceStartParagraph(s);
       break;
@@ -358,12 +391,15 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
       forceStartParagraph(s);
       break;
     case DocVerbatim::HtmlOnly: 
+      if (s->isBlock()) forceEndParagraph(s);
       m_t << s->text(); 
+      if (s->isBlock()) forceStartParagraph(s);
       break;
     case DocVerbatim::ManOnly: 
     case DocVerbatim::LatexOnly: 
     case DocVerbatim::XmlOnly: 
-    case DocVerbatim::RtfOnly: 
+    case DocVerbatim::RtfOnly:
+    case DocVerbatim::DocbookOnly:
       /* nothing */ 
       break;
 
@@ -372,6 +408,7 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         static int dotindex = 1;
         QCString fileName(4096);
 
+        forceEndParagraph(s);
         fileName.sprintf("%s%d%s", 
             (Config_getString("HTML_OUTPUT")+"/inline_dotgraph_").data(), 
             dotindex++,
@@ -382,45 +419,73 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         {
           err("Could not open file %s for writing\n",fileName.data());
         }
-        file.writeBlock( s->text(), s->text().length() );
-        file.close();
+        else
+        {
+          file.writeBlock( s->text(), s->text().length() );
+          file.close();
 
-        forceEndParagraph(s);
-        m_t << "<div align=\"center\">" << endl;
-        writeDotFile(fileName,s->relPath(),s->context());
-        m_t << "</div>" << endl;
+          m_t << "<div align=\"center\">" << endl;
+          writeDotFile(fileName,s->relPath(),s->context());
+          visitPreCaption(m_t, s);
+          visitCaption(this, s->children());
+          visitPostCaption(m_t, s);
+          m_t << "</div>" << endl;
+
+          if (Config_getBool("DOT_CLEANUP")) file.remove();
+        }
         forceStartParagraph(s);
-
-        if (Config_getBool("DOT_CLEANUP")) file.remove();
       }
       break;
     case DocVerbatim::Msc:
       {
+        forceEndParagraph(s);
+
         static int mscindex = 1;
         QCString baseName(4096);
 
         baseName.sprintf("%s%d", 
             (Config_getString("HTML_OUTPUT")+"/inline_mscgraph_").data(), 
             mscindex++
-           );
+            );
         QFile file(baseName+".msc");
         if (!file.open(IO_WriteOnly))
         {
           err("Could not open file %s.msc for writing\n",baseName.data());
         }
-        QCString text = "msc {";
-        text+=s->text();
-        text+="}";
-        file.writeBlock( text, text.length() );
-        file.close();
+        else
+        {
+          QCString text = "msc {";
+          text+=s->text();
+          text+="}";
 
+          file.writeBlock( text, text.length() );
+          file.close();
+
+          m_t << "<div align=\"center\">" << endl;
+          writeMscFile(baseName+".msc",s->relPath(),s->context());
+          visitPreCaption(m_t, s);
+          visitCaption(this, s->children());
+          visitPostCaption(m_t, s);
+          m_t << "</div>" << endl;
+
+          if (Config_getBool("DOT_CLEANUP")) file.remove();
+        }
+        forceStartParagraph(s);
+      }
+      break;
+    case DocVerbatim::PlantUML:
+      {
         forceEndParagraph(s);
+
+        static QCString htmlOutput = Config_getString("HTML_OUTPUT");
+        QCString baseName = writePlantUMLSource(htmlOutput,s->exampleFile(),s->text());
         m_t << "<div align=\"center\">" << endl;
-        writeMscFile(baseName+".msc",s->relPath(),s->context());
+        writePlantUMLFile(baseName,s->relPath(),s->context());
+        visitPreCaption(m_t, s);
+        visitCaption(this, s->children());
+        visitPostCaption(m_t, s);
         m_t << "</div>" << endl;
         forceStartParagraph(s);
-
-        if (Config_getBool("DOT_CLEANUP")) file.remove();
       }
       break;
   }
@@ -435,6 +500,7 @@ void HtmlDocVisitor::visit(DocAnchor *anc)
 void HtmlDocVisitor::visit(DocInclude *inc)
 {
   if (m_hide) return;
+  SrcLangExt langExt = getLanguageFromFileName(inc->extension());
   switch(inc->type())
   {
     case DocInclude::Include: 
@@ -444,12 +510,16 @@ void HtmlDocVisitor::visit(DocInclude *inc)
                             ->parseCode(m_ci,                 
                                         inc->context(),
                                         inc->text(),
+                                        langExt,
                                         inc->isExample(),
                                         inc->exampleFile(),
-                                        0,   // fd
-                                        -1,  // startLine
-                                        -1,  // endLine
-                                        TRUE // inlineFragment
+                                        0,     // fileDef
+                                        -1,    // startLine
+                                        -1,    // endLine
+                                        TRUE,  // inlineFragment
+                                        0,     // memberDef
+                                        FALSE, // show line numbers
+                                        m_ctx  // search context 
                                        );
       m_t << PREFRAG_END;
       forceStartParagraph(inc);
@@ -464,8 +534,17 @@ void HtmlDocVisitor::visit(DocInclude *inc)
                                ->parseCode(m_ci,
                                            inc->context(),
                                            inc->text(),
+                                           langExt,
                                            inc->isExample(),
-                                           inc->exampleFile(), &fd);
+                                           inc->exampleFile(), 
+                                           &fd,   // fileDef,
+                                           -1,    // start line
+                                           -1,    // end line
+                                           FALSE, // inline fragment
+                                           0,     // memberDef
+                                           TRUE,  // show line numbers
+                                           m_ctx  // search context
+                                           );
          m_t << PREFRAG_END;
          forceStartParagraph(inc);
       }
@@ -475,6 +554,8 @@ void HtmlDocVisitor::visit(DocInclude *inc)
     case DocInclude::HtmlInclude: 
       m_t << inc->text(); 
       break;
+    case DocInclude::LatexInclude:
+      break;
     case DocInclude::VerbInclude: 
       forceEndParagraph(inc);
       m_t << /*PREFRAG_START <<*/ "<pre class=\"fragment\">";
@@ -490,12 +571,16 @@ void HtmlDocVisitor::visit(DocInclude *inc)
                                ->parseCode(m_ci,
                                            inc->context(),
                                            extractBlock(inc->text(),inc->blockId()),
+                                           langExt,
                                            inc->isExample(),
                                            inc->exampleFile(), 
                                            0,
-                                           -1,  // startLine
-                                           -1,  // endLine
-                                           TRUE // inlineFragment
+                                           -1,    // startLine
+                                           -1,    // endLine
+                                           TRUE,  // inlineFragment
+                                           0,     // memberDef
+                                           TRUE,  // show line number
+                                           m_ctx  // search context
                                           );
          m_t << PREFRAG_END;
          forceStartParagraph(inc);
@@ -514,15 +599,28 @@ void HtmlDocVisitor::visit(DocIncOperator *op)
     pushEnabled();
     m_hide=TRUE;
   }
+  SrcLangExt langExt = getLanguageFromFileName(m_langExt);
   if (op->type()!=DocIncOperator::Skip) 
   {
     popEnabled();
     if (!m_hide) 
     {
       Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,op->context(),
-                                op->text(),op->isExample(),
-                                op->exampleFile());
+                            ->parseCode(
+                                m_ci,
+                                op->context(),
+                                op->text(),
+                                langExt,
+                                op->isExample(),
+                                op->exampleFile(),
+                                0,     // fileDef
+                                -1,    // startLine
+                                -1,    // endLine
+                                FALSE, // inline fragment
+                                0,     // memberDef
+                                TRUE,  // show line numbers
+                                m_ctx  // search context
+                               );
     }
     pushEnabled();
     m_hide=TRUE;
@@ -597,7 +695,7 @@ void HtmlDocVisitor::visit(DocIndexEntry *e)
   //       e->scope()  ? e->scope()->name().data()  : "<null>",
   //       e->member() ? e->member()->name().data() : "<null>"
   //      );
-  Doxygen::indexList.addIndexItem(e->scope(),e->member(),e->entry());
+  Doxygen::indexList->addIndexItem(e->scope(),e->member(),anchor,e->entry());
 }
 
 void HtmlDocVisitor::visit(DocSimpleSectSep *)
@@ -705,21 +803,21 @@ bool isSeparatedParagraph(DocSimpleSect *parent,DocPara *par)
   int i = nodes.findRef(par);
   if (i==-1) return FALSE;
   int count = parent->children().count();
-  if (count>1 && i==0)
+  if (count>1 && i==0) // first node
   {
     if (nodes.at(i+1)->kind()==DocNode::Kind_SimpleSectSep)
     {
       return TRUE;
     }
   }
-  else if (count>1 && i==count-1)
+  else if (count>1 && i==count-1) // last node
   {
     if (nodes.at(i-1)->kind()==DocNode::Kind_SimpleSectSep)
     {
       return TRUE;
     }
   }
-  else if (count>2 && i>0 && i<count-1)
+  else if (count>2 && i>0 && i<count-1) // intermediate node
   {
     if (nodes.at(i-1)->kind()==DocNode::Kind_SimpleSectSep &&
         nodes.at(i+1)->kind()==DocNode::Kind_SimpleSectSep)
@@ -739,9 +837,58 @@ static int getParagraphContext(DocPara *p,bool &isFirst,bool &isLast)
   {
     switch (p->parent()->kind()) 
     {
+      case DocNode::Kind_ParBlock:
+        { // hierarchy: node N -> para -> parblock -> para
+          // adapt return value to kind of N
+          DocNode::Kind kind = DocNode::Kind_Para;
+          if ( p->parent()->parent() && p->parent()->parent()->parent() )
+          {
+            kind = p->parent()->parent()->parent()->kind();
+          }
+          isFirst=isFirstChildNode((DocParBlock*)p->parent(),p);
+          isLast =isLastChildNode ((DocParBlock*)p->parent(),p);
+          t=0;
+          if (isFirst)
+          {
+            if (kind==DocNode::Kind_HtmlListItem ||
+                kind==DocNode::Kind_SecRefItem)
+            {
+              t=1;
+            }
+            else if (kind==DocNode::Kind_HtmlDescData ||
+                     kind==DocNode::Kind_XRefItem ||
+                     kind==DocNode::Kind_SimpleSect)
+            {
+              t=2;
+            }
+            else if (kind==DocNode::Kind_HtmlCell ||
+                     kind==DocNode::Kind_ParamList)
+            {
+              t=5;
+            }
+          }
+          if (isLast)
+          {
+            if (kind==DocNode::Kind_HtmlListItem ||
+                kind==DocNode::Kind_SecRefItem)
+            {
+              t=3;
+            }
+            else if (kind==DocNode::Kind_HtmlDescData ||
+                     kind==DocNode::Kind_XRefItem ||
+                     kind==DocNode::Kind_SimpleSect)
+            {
+              t=4;
+            }
+            else if (kind==DocNode::Kind_HtmlCell ||
+                     kind==DocNode::Kind_ParamList)
+            {
+              t=6;
+            }
+          }
+          break;
+        }
       case DocNode::Kind_AutoListItem:
-        //isFirst=TRUE;
-        //isLast =TRUE;
         isFirst=isFirstChildNode((DocAutoListItem*)p->parent(),p);
         isLast =isLastChildNode ((DocAutoListItem*)p->parent(),p);
         t=1; // not used
@@ -780,12 +927,6 @@ static int getParagraphContext(DocPara *p,bool &isFirst,bool &isLast)
         if (isFirst) t=2;
         if (isLast)  t=4;
         break;
-      case DocNode::Kind_HtmlCell:
-        isFirst=isFirstChildNode((DocHtmlCell*)p->parent(),p);
-        isLast =isLastChildNode ((DocHtmlCell*)p->parent(),p);
-        if (isFirst) t=5;
-        if (isLast)  t=6;
-        break;
       case DocNode::Kind_SimpleSect:
         isFirst=isFirstChildNode((DocSimpleSect*)p->parent(),p);
         isLast =isLastChildNode ((DocSimpleSect*)p->parent(),p);
@@ -799,6 +940,12 @@ static int getParagraphContext(DocPara *p,bool &isFirst,bool &isLast)
           isFirst=isLast=TRUE;
         }
         break;
+      case DocNode::Kind_HtmlCell:
+        isFirst=isFirstChildNode((DocHtmlCell*)p->parent(),p);
+        isLast =isLastChildNode ((DocHtmlCell*)p->parent(),p);
+        if (isFirst) t=5;
+        if (isLast)  t=6;
+        break;
       default:
         break;
     }
@@ -831,6 +978,7 @@ void HtmlDocVisitor::visitPre(DocPara *p)
       case DocNode::Kind_XRefItem:
       case DocNode::Kind_Copy:
       case DocNode::Kind_HtmlBlockQuote:
+      case DocNode::Kind_ParBlock:
         needsTag = TRUE;
         break;
       case DocNode::Kind_Root:
@@ -904,6 +1052,7 @@ void HtmlDocVisitor::visitPost(DocPara *p)
       case DocNode::Kind_XRefItem:
       case DocNode::Kind_Copy:
       case DocNode::Kind_HtmlBlockQuote:
+      case DocNode::Kind_ParBlock:
         needsTag = TRUE;
         break;
       case DocNode::Kind_Root:
@@ -914,7 +1063,6 @@ void HtmlDocVisitor::visitPost(DocPara *p)
     }
   }
 
-  QCString context;
   // if the last element of a paragraph is something that should be outside of
   // the paragraph (<ul>,<dl>,<table>) then that will already have ended the 
   // paragraph and we don't need to do it here
@@ -1000,7 +1148,7 @@ void HtmlDocVisitor::visitPre(DocSimpleSect *s)
   // special case 1: user defined title
   if (s->type()!=DocSimpleSect::User && s->type()!=DocSimpleSect::Rcs)
   {
-    m_t << ":</dt><dd>";
+    m_t << "</dt><dd>";
   }
 }
 
@@ -1369,6 +1517,26 @@ void HtmlDocVisitor::visitPost(DocMscFile *df)
   m_t << "</div>" << endl;
 }
 
+void HtmlDocVisitor::visitPre(DocDiaFile *df)
+{
+  if (m_hide) return;
+  m_t << "<div class=\"diagraph\">" << endl;
+  writeDiaFile(df->file(),df->relPath(),df->context());
+  if (df->hasCaption())
+  {
+    m_t << "<div class=\"caption\">" << endl;
+  }
+}
+void HtmlDocVisitor::visitPost(DocDiaFile *df)
+{
+  if (m_hide) return;
+  if (df->hasCaption())
+  {
+    m_t << "</div>" << endl;
+  }
+  m_t << "</div>" << endl;
+}
+
 void HtmlDocVisitor::visitPre(DocLink *lnk)
 {
   if (m_hide) return;
@@ -1482,7 +1650,7 @@ void HtmlDocVisitor::visitPre(DocParamSect *s)
       ASSERT(0);
   }
   m_t << "<dl class=\"" << className << "\"><dt>";
-  m_t << heading << ":";
+  m_t << heading;
   m_t << "</dt><dd>" << endl;
   m_t << "  <table class=\"" << className << "\">" << endl;
 }
@@ -1579,6 +1747,8 @@ void HtmlDocVisitor::visitPost(DocParamList *)
 void HtmlDocVisitor::visitPre(DocXRefItem *x)
 {
   if (m_hide) return;
+  if (x->title().isEmpty()) return;
+
   forceEndParagraph(x);
   bool anonymousEnum = x->file()=="@";
   if (!anonymousEnum)
@@ -1600,6 +1770,7 @@ void HtmlDocVisitor::visitPre(DocXRefItem *x)
 void HtmlDocVisitor::visitPost(DocXRefItem *x)
 {
   if (m_hide) return;
+  if (x->title().isEmpty()) return;
   m_t << "</dd></dl>" << endl;
   forceStartParagraph(x);
 }
@@ -1656,6 +1827,49 @@ void HtmlDocVisitor::visitPost(DocHtmlBlockQuote *b)
   forceStartParagraph(b);
 }
 
+void HtmlDocVisitor::visitPre(DocVhdlFlow *vf)
+{
+  if (m_hide) return;
+  if (VhdlDocGen::getFlowMember()) // use VHDL flow chart creator
+  {
+    forceEndParagraph(vf);
+    QCString fname=FlowChart::convertNameToFileName();
+    m_t << "<p>";
+    m_t << "flowchart: " ; // TODO: translate me
+    m_t << "<a href=\"";
+    m_t << fname.data(); 
+    m_t << ".svg\">";
+    m_t << VhdlDocGen::getFlowMember()->name().data();
+    m_t << "</a>";
+    if (vf->hasCaption())
+    {
+      m_t << "<br />";
+    }
+  }
+}
+
+void HtmlDocVisitor::visitPost(DocVhdlFlow *vf)
+{
+  if (m_hide) return;
+  if (VhdlDocGen::getFlowMember()) // use VHDL flow chart creator
+  {
+    m_t << "</p>";
+    forceStartParagraph(vf);
+  }
+}
+
+void HtmlDocVisitor::visitPre(DocParBlock *)
+{
+  if (m_hide) return;
+}
+
+void HtmlDocVisitor::visitPost(DocParBlock *)
+{
+  if (m_hide) return;
+}
+
+
+
 void HtmlDocVisitor::filter(const char *str)
 { 
   if (str==0) return;
@@ -1692,11 +1906,11 @@ void HtmlDocVisitor::filterQuotedCdataAttr(const char* str)
        // it's enough to replace these two, provided that the declaration
        // for the HTML version we generate (and as supported by the browser)
        // specifies that all the other symbols used in rawVal are
-       // within the right charachter class (i.e., they're not
-       // some multinational weird charachters not in the BASESET).
+       // within the right character class (i.e., they're not
+       // some multinational weird characters not in the BASESET).
        // We assume that 1) the browser will support whatever is remaining
        // in the formula and 2) the TeX formulae are generally governed
-       // by even stricter charachter restrictions so it should be enough.
+       // by even stricter character restrictions so it should be enough.
        //
        // On some incompliant browsers, additional translation of
        // '>' and '<' into "&gt;" and "&lt;", respectively, might be needed;
@@ -1715,6 +1929,7 @@ void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,
                                const QCString &relPath,const QCString &anchor,
                                const QCString &tooltip)
 {
+  //printf("HtmlDocVisitor: file=%s anchor=%s\n",file.data(),anchor.data());
   if (!ref.isEmpty()) // link to entity imported via tag file
   {
     m_t << "<a class=\"elRef\" ";
@@ -1766,7 +1981,7 @@ void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
   }
   baseName.prepend("dot_");
   QCString outDir = Config_getString("HTML_OUTPUT");
-  writeDotGraphFromFile(fn,outDir,baseName,BITMAP);
+  writeDotGraphFromFile(fn,outDir,baseName,GOF_BITMAP);
   writeDotImageMapFromFile(m_t,fn,outDir,relPath,baseName,context);
 }
 
@@ -1786,8 +2001,64 @@ void HtmlDocVisitor::writeMscFile(const QCString &fileName,
   }
   baseName.prepend("msc_");
   QCString outDir = Config_getString("HTML_OUTPUT");
-  writeMscGraphFromFile(fileName,outDir,baseName,MSC_BITMAP);
-  writeMscImageMapFromFile(m_t,fileName,outDir,relPath,baseName,context);
+  QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
+  MscOutputFormat mscFormat = MSC_BITMAP;
+  if ("svg" == imgExt)
+    mscFormat = MSC_SVG;
+  writeMscGraphFromFile(fileName,outDir,baseName,mscFormat);
+  writeMscImageMapFromFile(m_t,fileName,outDir,relPath,baseName,context,mscFormat);
+}
+
+void HtmlDocVisitor::writeDiaFile(const QCString &fileName,
+                                  const QCString &relPath,
+                                  const QCString &)
+{
+  QCString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1) // strip path
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  }
+  if ((i=baseName.find('.'))!=-1) // strip extension
+  {
+    baseName=baseName.left(i);
+  }
+  baseName.prepend("dia_");
+  QCString outDir = Config_getString("HTML_OUTPUT");
+  writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP);
+
+  m_t << "<img src=\"" << relPath << baseName << ".png" << "\" />" << endl;
+}
+
+void HtmlDocVisitor::writePlantUMLFile(const QCString &fileName,
+                                       const QCString &relPath,
+                                       const QCString &)
+{
+  QCString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1) // strip path
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  }
+  if ((i=baseName.findRev('.'))!=-1) // strip extension
+  {
+    baseName=baseName.left(i);
+  }
+  static QCString outDir = Config_getString("HTML_OUTPUT");
+  static QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
+  if (imgExt=="svg")
+  {
+    generatePlantUMLOutput(fileName,outDir,PUML_SVG);
+    //m_t << "<iframe scrolling=\"no\" frameborder=\"0\" src=\"" << relPath << baseName << ".svg" << "\" />" << endl;
+    //m_t << "<p><b>This browser is not able to show SVG: try Firefox, Chrome, Safari, or Opera instead.</b></p>";
+    //m_t << "</iframe>" << endl;
+    m_t << "<object type=\"image/svg+xml\" data=\"" << relPath << baseName << ".svg\"></object>" << endl;
+  }
+  else
+  {
+    generatePlantUMLOutput(fileName,outDir,PUML_BITMAP);
+    m_t << "<img src=\"" << relPath << baseName << ".png" << "\" />" << endl;
+  }
 }
 
 /** Used for items found inside a paragraph, which due to XHTML restrictions
@@ -1822,7 +2093,7 @@ void HtmlDocVisitor::forceEndParagraph(DocNode *n)
     //printf("forceEnd first=%d last=%d\n",isFirst,isLast);
     if (isFirst && isLast) return;
 
-    m_t << "</p>" << endl;
+    m_t << "</p>";
   }
 }
 
