@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * 
  *
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -23,6 +23,8 @@
 #include <qstack.h>
 #include <qcstring.h>
 
+class Definition;
+class MemberDef;
 class DocNode;
 class FTextStream;
 class CodeOutputInterface;
@@ -31,7 +33,7 @@ class CodeOutputInterface;
 class HtmlDocVisitor : public DocVisitor
 {
   public:
-    HtmlDocVisitor(FTextStream &t,CodeOutputInterface &ci,const char *langExt);
+    HtmlDocVisitor(FTextStream &t,CodeOutputInterface &ci,Definition *ctx);
     
     //--------------------------------------
     // visitor functions for leaf nodes
@@ -52,6 +54,7 @@ class HtmlDocVisitor : public DocVisitor
     void visit(DocFormula *);
     void visit(DocIndexEntry *);
     void visit(DocSimpleSectSep *);
+    void visit(DocCite *);
 
     //--------------------------------------
     // visitor functions for compound nodes
@@ -105,6 +108,8 @@ class HtmlDocVisitor : public DocVisitor
     void visitPost(DocDotFile *);
     void visitPre(DocMscFile *);
     void visitPost(DocMscFile *);
+    void visitPre(DocDiaFile *);
+    void visitPost(DocDiaFile *);
     void visitPre(DocLink *);
     void visitPost(DocLink *);
     void visitPre(DocRef *);
@@ -125,6 +130,12 @@ class HtmlDocVisitor : public DocVisitor
     void visitPost(DocCopy *);
     void visitPre(DocText *);
     void visitPost(DocText *);
+    void visitPre(DocHtmlBlockQuote *);
+    void visitPost(DocHtmlBlockQuote *);
+    void visitPre(DocVhdlFlow *);
+    void visitPost(DocVhdlFlow *);
+    void visitPre(DocParBlock *);
+    void visitPost(DocParBlock *);
 
   private:
 
@@ -132,6 +143,7 @@ class HtmlDocVisitor : public DocVisitor
     // helper functions 
     //--------------------------------------
     
+    void writeObfuscatedMailAddress(const QCString &url);
     void filter(const char *str);
     void filterQuotedCdataAttr(const char* str);
     void startLink(const QCString &ref,const QCString &file,
@@ -140,6 +152,8 @@ class HtmlDocVisitor : public DocVisitor
     void endLink();
     void writeDotFile(const QCString &fileName,const QCString &relPath,const QCString &context);
     void writeMscFile(const QCString &fileName,const QCString &relPath,const QCString &context);
+    void writeDiaFile(const QCString &fileName,const QCString &relPath,const QCString &context);
+    void writePlantUMLFile(const QCString &fileName,const QCString &relPath,const QCString &context);
 
     void pushEnabled();
     void popEnabled();
@@ -156,6 +170,7 @@ class HtmlDocVisitor : public DocVisitor
     bool m_insidePre;
     bool m_hide;
     QStack<bool> m_enabled;
+    Definition *m_ctx;
     QCString m_langExt;
 };
 
