@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * 
  *
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,6 +15,7 @@
  * input used in their production; they are not affected by this license.
  *
  */
+#include "htmlattrib.h"
 #include <qfileinfo.h> 
 #include "latexdocvisitor.h"
 #include "docparser.h"
@@ -26,22 +27,32 @@
 #include "message.h"
 #include "parserintf.h"
 #include "msc.h"
-#include "htmlattrib.h"
+#include "dia.h"
 #include "cite.h"
+#include "filedef.h"
+#include "config.h"
+#include "htmlentity.h"
+#include "plantuml.h"
 
 static QCString escapeLabelName(const char *s)
 {
   QCString result;
   const char *p=s;
   char c;
-  while ((c=*p++))
+  if (p)
   {
-    switch (c)
+    while ((c=*p++))
     {
-      case '%': result+="\\%"; break;
-      case '|': result+="\\texttt{\"|}"; break;
-      case '!': result+="\"!"; break;
-      default: result+=c;
+      switch (c)
+      {
+        case '%': result+="\\%"; break;
+        case '|': result+="\\texttt{\"|}"; break;
+        case '!': result+="\"!"; break;
+        case '{': result+="\\lcurly{}"; break;
+        case '}': result+="\\rcurly{}"; break;
+        case '~': result+="````~"; break; // to get it a bit better in index together with other special characters
+        default: result+=c;
+      }
     }
   }
   return result;
@@ -60,23 +71,97 @@ static const char *getSectionName(int level)
   return secLabels[QMIN(maxLevels-1,l)];
 }
 
+static void visitPreStart(FTextStream &t, const bool hasCaption, QCString name,  QCString width,  QCString height)
+{
+    if (hasCaption)
+    {
+      t << "\n\\begin{DoxyImage}\n";
+    }
+    else
+    {
+      t << "\n\\begin{DoxyImageNoCaption}\n"
+             "  \\mbox{";
+    }
+
+    t << "\\includegraphics";
+    if (!width.isEmpty() || !height.isEmpty())
+    {
+      t << "[";
+    }
+    if (!width.isEmpty())
+    {
+      t << "width=" << width;
+    }
+    if (!width.isEmpty() && !height.isEmpty())
+    {
+      t << ",";
+    }
+    if (!height.isEmpty())
+    {
+      t << "height=" << height;
+    }
+    if (width.isEmpty() && height.isEmpty())
+    {
+      /* default setting */
+      t << "[width=\\textwidth,height=\\textheight/2,keepaspectratio=true]";
+    }
+    else
+    {
+      t << "]";
+    }
+
+    t << "{" << name << "}";
+
+    if (hasCaption)
+    {
+      t << "\n\\caption{";
+    }
+}
+
+
+
+static void visitPostEnd(FTextStream &t, const bool hasCaption)
+{
+    t << "}\n"; // end mbox or caption
+    if (hasCaption)
+    {
+      t << "\\end{DoxyImage}\n";
+    }
+    else{
+      t << "\\end{DoxyImageNoCaption}\n";
+    }
+}
+
+
+static void visitCaption(LatexDocVisitor *parent, QList<DocNode> children)
+{
+  QListIterator<DocNode> cli(children);
+  DocNode *n;
+  for (cli.toFirst();(n=cli.current());++cli) n->accept(parent);
+}
+
 QCString LatexDocVisitor::escapeMakeIndexChars(const char *s)
 {
   QCString result;
   const char *p=s;
   char str[2]; str[1]=0;
   char c;
-  while ((c=*p++))
+  if (p)
   {
-    switch (c)
+    while ((c=*p++))
     {
-      case '!': m_t << "\"!"; break;
-      case '"': m_t << "\"\""; break;
-      case '@': m_t << "\"@"; break;
-      case '|': m_t << "\\texttt{\"|}"; break;
-      case '[': m_t << "["; break;
-      case ']': m_t << "]"; break;
-      default:  str[0]=c; filter(str); break;
+      switch (c)
+      {
+        case '!': m_t << "\"!"; break;
+        case '"': m_t << "\"\""; break;
+        case '@': m_t << "\"@"; break;
+        case '|': m_t << "\\texttt{\"|}"; break;
+        case '[': m_t << "["; break;
+        case ']': m_t << "]"; break;
+        case '{': m_t << "\\lcurly{}"; break;
+        case '}': m_t << "\\rcurly{}"; break;
+        default:  str[0]=c; filter(str); break;
+      }
     }
   }
   return result;
@@ -127,67 +212,25 @@ void LatexDocVisitor::visit(DocWhiteSpace *w)
 void LatexDocVisitor::visit(DocSymbol *s)
 {
   if (m_hide) return;
-  switch(s->symbol())
+  const char *res = HtmlEntityMapper::instance()->latex(s->symbol());
+  if (res)
   {
-    case DocSymbol::BSlash:  m_t << "\\textbackslash{}"; break;
-    case DocSymbol::At:      m_t << "@"; break;
-    case DocSymbol::Less:    if (m_insidePre) m_t << "<"; else m_t << "$<$"; 
-                             break;
-    case DocSymbol::Greater: if (m_insidePre) m_t << ">"; else m_t << "$>$"; break;
-    case DocSymbol::Amp:     m_t << "\\&"; break;
-    case DocSymbol::Dollar:  m_t << "\\$"; break;
-    case DocSymbol::Hash:    m_t << "\\#"; break;
-    case DocSymbol::DoubleColon:  m_t << "::"; break;
-    case DocSymbol::Percent: m_t << "\\%"; break;
-    case DocSymbol::Copy:    m_t << "\\copyright"; break;
-    case DocSymbol::Tm:      m_t << "\\texttrademark"; break;
-    case DocSymbol::Reg:     m_t << "\\textregistered"; break;
-    case DocSymbol::Apos:    m_t << "'"; break;
-    case DocSymbol::Quot:    m_t << "\""; break;
-    case DocSymbol::Lsquo:   m_t << "`"; break;
-    case DocSymbol::Rsquo:   m_t << "'"; break;
-    case DocSymbol::Ldquo:   m_t << "``"; break;
-    case DocSymbol::Rdquo:   m_t << "''"; break;
-    case DocSymbol::Ndash:   m_t << "--"; break;
-    case DocSymbol::Mdash:   m_t << "---"; break;
-    case DocSymbol::Uml:     
-                             if (s->letter()=='i') 
-                               m_t << "\\\"{\\i}"; 
-                             else                  
-                               m_t << "\\\"{" << s->letter() << "}"; 
-                             break;
-    case DocSymbol::Acute:   
-                             if (s->letter()=='i') 
-                               m_t << "\\'{\\i}"; 
-                             else                  
-                               m_t << "\\'{" << s->letter() << "}"; 
-                             break;
-    case DocSymbol::Grave:   
-                             if (s->letter()=='i') 
-                               m_t << "\\`{\\i}"; 
-                             else                  
-                               m_t << "\\`{" << s->letter() << "}"; 
-                             break;
-    case DocSymbol::Circ:    
-                             if (s->letter()=='i') 
-                               m_t << "\\^{\\i}"; 
-                             else                  
-                               m_t << "\\^{" << s->letter() << "}"; 
-                             break;
-    case DocSymbol::Slash:   if (tolower(s->letter())=='o')
-                               m_t << "{\\" << s->letter() << "}";
-                             else
-                               m_t << s->letter();
-                             break;
-    case DocSymbol::Tilde:   m_t << "\\~{"  << s->letter() << "}"; break;
-    case DocSymbol::Szlig:   m_t << "{\\ss}"; break;
-    case DocSymbol::Cedil:   m_t << "\\c{" << s->letter() << "}"; break;
-    case DocSymbol::Ring:    m_t << "\\" << s->letter() << s->letter(); break;
-    case DocSymbol::Nbsp:    m_t << "~"; break;
-    case DocSymbol::AElig:   m_t << "{\\AE}"; break;
-    case DocSymbol::Aelig:   m_t << "{\\ae}"; break;
-    default:
-                             err("error: unknown symbol found\n");
+    if (((s->symbol() == DocSymbol::Sym_lt) || (s->symbol() == DocSymbol::Sym_Less))&& (!m_insidePre))
+    {
+      m_t << "$<$";
+    }
+    else if (((s->symbol() == DocSymbol::Sym_gt) || (s->symbol() == DocSymbol::Sym_Greater)) && (!m_insidePre))
+    {
+      m_t << "$>$";
+    }
+    else
+    {
+      m_t << res;
+    }
+  }
+  else
+  {
+    err("LaTeX: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance()->html(s->symbol(),TRUE));
   }
 }
 
@@ -208,8 +251,7 @@ void LatexDocVisitor::visit(DocURL *u)
 void LatexDocVisitor::visit(DocLineBreak *)
 {
   if (m_hide) return;
-  if (m_insideTable) m_t << "\\newline\n";
-  else m_t << "\\par\n";
+  m_t << "~\\newline\n";
 }
 
 void LatexDocVisitor::visit(DocHorRuler *)
@@ -233,10 +275,10 @@ void LatexDocVisitor::visit(DocStyleChange *s)
       if (s->enable()) m_t << "{\\ttfamily ";   else m_t << "}";
       break;
     case DocStyleChange::Subscript:
-      if (s->enable()) m_t << "$_{\\mbox{";    else m_t << "}}$ ";
+      if (s->enable()) m_t << "\\textsubscript{";    else m_t << "}";
       break;
     case DocStyleChange::Superscript:
-      if (s->enable()) m_t << "$^{\\mbox{";    else m_t << "}}$ ";
+      if (s->enable()) m_t << "\\textsuperscript{";    else m_t << "}";
       break;
     case DocStyleChange::Center:
       if (s->enable()) m_t << "\\begin{center}"; else m_t << "\\end{center} ";
@@ -269,13 +311,14 @@ void LatexDocVisitor::visit(DocVerbatim *s)
   {
     lang = s->language();
   }
+  SrcLangExt langExt = getLanguageFromFileName(lang);
   switch(s->type())
   {
     case DocVerbatim::Code: 
       {
         m_t << "\n\\begin{DoxyCode}\n";
         Doxygen::parserManager->getParser(lang)
-                              ->parseCode(m_ci,s->context(),s->text(),
+                              ->parseCode(m_ci,s->context(),s->text(),langExt,
                                           s->isExample(),s->exampleFile());
         m_t << "\\end{DoxyCode}\n";
       }
@@ -288,7 +331,8 @@ void LatexDocVisitor::visit(DocVerbatim *s)
     case DocVerbatim::HtmlOnly: 
     case DocVerbatim::XmlOnly: 
     case DocVerbatim::ManOnly: 
-    case DocVerbatim::RtfOnly: 
+    case DocVerbatim::RtfOnly:
+    case DocVerbatim::DocbookOnly:
       /* nothing */ 
       break;
     case DocVerbatim::LatexOnly: 
@@ -309,15 +353,17 @@ void LatexDocVisitor::visit(DocVerbatim *s)
         {
           err("Could not open file %s for writing\n",fileName.data());
         }
-        file.writeBlock( s->text(), s->text().length() );
-        file.close();
+        else
+        {
+          file.writeBlock( s->text(), s->text().length() );
+          file.close();
 
-        m_t << "\\begin{center}\n";
-        startDotFile(fileName,"","",FALSE);
-        endDotFile(FALSE);
-        m_t << "\\end{center}\n";
+          startDotFile(fileName,s->width(),s->height(),s->hasCaption());
+          visitCaption(this, s->children());
+          endDotFile(s->hasCaption());
 
-        if (Config_getBool("DOT_CLEANUP")) file.remove();
+          if (Config_getBool("DOT_CLEANUP")) file.remove();
+        }
       }
       break;
     case DocVerbatim::Msc: 
@@ -334,17 +380,26 @@ void LatexDocVisitor::visit(DocVerbatim *s)
         {
           err("Could not open file %s.msc for writing\n",baseName.data());
         }
-        QCString text = "msc {";
-        text+=s->text();
-        text+="}";
-        file.writeBlock( text, text.length() );
-        file.close();
+        else
+        {
+          QCString text = "msc {";
+          text+=s->text();
+          text+="}";
+          file.writeBlock( text, text.length() );
+          file.close();
 
-        m_t << "\\begin{center}\n";
-        writeMscFile(baseName);
-        m_t << "\\end{center}\n";
+          writeMscFile(baseName, s);
 
-        if (Config_getBool("DOT_CLEANUP")) file.remove();
+          if (Config_getBool("DOT_CLEANUP")) file.remove();
+        }
+      }
+      break;
+    case DocVerbatim::PlantUML: 
+      {
+        QCString latexOutput = Config_getString("LATEX_OUTPUT");
+        QCString baseName = writePlantUMLSource(latexOutput,s->exampleFile(),s->text());
+
+        writePlantUMLFile(baseName, s);
       }
       break;
   }
@@ -353,10 +408,10 @@ void LatexDocVisitor::visit(DocVerbatim *s)
 void LatexDocVisitor::visit(DocAnchor *anc)
 {
   if (m_hide) return;
-  m_t << "\\label{" << anc->file() << "_" << anc->anchor() << "}%" << endl;
+  m_t << "\\label{" << stripPath(anc->file()) << "_" << anc->anchor() << "}%" << endl;
   if (!anc->file().isEmpty() && Config_getBool("PDF_HYPERLINKS")) 
   {
-    m_t << "\\hypertarget{" << anc->file() << "_" << anc->anchor() 
+    m_t << "\\hypertarget{" << stripPath(anc->file()) << "_" << anc->anchor() 
       << "}{}%" << endl;
   }    
 }
@@ -364,6 +419,7 @@ void LatexDocVisitor::visit(DocAnchor *anc)
 void LatexDocVisitor::visit(DocInclude *inc)
 {
   if (m_hide) return;
+  SrcLangExt langExt = getLanguageFromFileName(inc->extension());
   switch(inc->type())
   {
     case DocInclude::IncWithLines:
@@ -374,6 +430,7 @@ void LatexDocVisitor::visit(DocInclude *inc)
          Doxygen::parserManager->getParser(inc->extension())
                                ->parseCode(m_ci,inc->context(),
                                            inc->text(),
+                                           langExt,
                                            inc->isExample(),
                                            inc->exampleFile(), &fd);
          m_t << "\\end{DoxyCodeInclude}" << endl;
@@ -383,7 +440,7 @@ void LatexDocVisitor::visit(DocInclude *inc)
       m_t << "\n\\begin{DoxyCodeInclude}\n";
       Doxygen::parserManager->getParser(inc->extension())
                             ->parseCode(m_ci,inc->context(),
-                                        inc->text(),inc->isExample(),
+                                        inc->text(),langExt,inc->isExample(),
                                         inc->exampleFile());
       m_t << "\\end{DoxyCodeInclude}\n";
       break;
@@ -391,6 +448,9 @@ void LatexDocVisitor::visit(DocInclude *inc)
       break;
     case DocInclude::HtmlInclude: 
       break;
+    case DocInclude::LatexInclude:
+      m_t << inc->text();
+      break;
     case DocInclude::VerbInclude: 
       m_t << "\n\\begin{DoxyVerbInclude}\n";
       m_t << inc->text();
@@ -403,6 +463,7 @@ void LatexDocVisitor::visit(DocInclude *inc)
                                ->parseCode(m_ci,
                                            inc->context(),
                                            extractBlock(inc->text(),inc->blockId()),
+                                           langExt,
                                            inc->isExample(),
                                            inc->exampleFile()
                                           );
@@ -422,13 +483,14 @@ void LatexDocVisitor::visit(DocIncOperator *op)
     pushEnabled();
     m_hide = TRUE;
   }
+  SrcLangExt langExt = getLanguageFromFileName(m_langExt);
   if (op->type()!=DocIncOperator::Skip) 
   {
     popEnabled();
     if (!m_hide) 
     {
       Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,op->context(),op->text(),
+                            ->parseCode(m_ci,op->context(),op->text(),langExt,
                                         op->isExample(),op->exampleFile());
     }
     pushEnabled();
@@ -727,11 +789,11 @@ void LatexDocVisitor::visitPre(DocSection *s)
   if (m_hide) return;
   if (Config_getBool("PDF_HYPERLINKS"))
   {
-    m_t << "\\hypertarget{" << s->file() << "_" << s->anchor() << "}{}";
+    m_t << "\\hypertarget{" << stripPath(s->file()) << "_" << s->anchor() << "}{}";
   }
   m_t << "\\" << getSectionName(s->level()) << "{";
   filter(convertCharEntitiesToUTF8(s->title().data()));
-  m_t << "}\\label{" << s->file() << "_" << s->anchor() << "}" << endl;
+  m_t << "}\\label{" << stripPath(s->file()) << "_" << s->anchor() << "}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocSection *) 
@@ -828,21 +890,32 @@ void LatexDocVisitor::visitPost(DocHtmlDescData *)
 {
 }
 
+static const char *getTableName(const DocNode *n)
+{
+  bool isNested=FALSE;
+  while (n && !isNested)
+  {
+    isNested = n->kind()==DocNode::Kind_HtmlTable || n->kind()==DocNode::Kind_ParamSect;
+    n  = n->parent();
+  }
+  return isNested ? "TabularNC" : "TabularC";
+}
+
 void LatexDocVisitor::visitPre(DocHtmlTable *t)
 {
   m_rowSpans.clear();
   m_insideTable=TRUE;
   if (m_hide) return;
-  if (t->hasCaption()) 
+  if (t->hasCaption())
   {
     m_t << "\\begin{table}[h]";
   }
-  m_t << "\\begin{TabularC}{" << t->numColumns() << "}\n";
+  m_t << "\\begin{" << getTableName(t->parent()) << "}{" << t->numColumns() << "}\n";
   m_numCols = t->numColumns();
   m_t << "\\hline\n";
 }
 
-void LatexDocVisitor::visitPost(DocHtmlTable *t) 
+void LatexDocVisitor::visitPost(DocHtmlTable *t)
 {
   m_insideTable=FALSE;
   if (m_hide) return;
@@ -852,14 +925,14 @@ void LatexDocVisitor::visitPost(DocHtmlTable *t)
   }
   else
   {
-    m_t << "\\end{TabularC}\n";
+    m_t << "\\end{" << getTableName(t->parent()) << "}\n";
   }
 }
 
-void LatexDocVisitor::visitPre(DocHtmlCaption *)
+void LatexDocVisitor::visitPre(DocHtmlCaption *c)
 {
   if (m_hide) return;
-  m_t << "\\end{TabularC}\n\\centering\n\\caption{";
+  m_t << "\\end{" << getTableName(c->parent()) << "}\n\\centering\n\\caption{";
 }
 
 void LatexDocVisitor::visitPost(DocHtmlCaption *) 
@@ -1095,40 +1168,18 @@ void LatexDocVisitor::visitPost(DocHtmlHeader *)
   if (m_hide) return;
   m_t << "}";
 }
-
 void LatexDocVisitor::visitPre(DocImage *img)
 {
   if (img->type()==DocImage::Latex)
   {
     if (m_hide) return;
-    if (img->hasCaption())
-    {
-      m_t << "\n\\begin{DoxyImage}\n";
-    }
-    else
-    {
-      m_t << "\n\\begin{DoxyImageNoCaption}\n"
-             "  \\mbox{";
-    }
     QCString gfxName = img->name();
     if (gfxName.right(4)==".eps" || gfxName.right(4)==".pdf")
     {
       gfxName=gfxName.left(gfxName.length()-4);
     }
-    m_t << "\\includegraphics";
-    if (!img->width().isEmpty())
-    {
-      m_t << "[width=" << img->width() << "]";
-    }
-    else if (!img->height().isEmpty())
-    {
-      m_t << "[height=" << img->height() << "]";
-    }
-    m_t << "{" << gfxName << "}";
-    if (img->hasCaption())
-    {
-      m_t << "\n\\caption{";
-    }
+
+    visitPreStart(m_t,img->hasCaption(), gfxName, img->width(),  img->height());
   }
   else // other format -> skip
   {
@@ -1142,14 +1193,7 @@ void LatexDocVisitor::visitPost(DocImage *img)
   if (img->type()==DocImage::Latex)
   {
     if (m_hide) return;
-    m_t << "}\n"; // end mbox or caption
-    if (img->hasCaption())
-    {
-      m_t << "\\end{DoxyImage}\n";
-    }
-    else{
-      m_t << "\\end{DoxyImageNoCaption}\n";
-    }
+    visitPostEnd(m_t,img->hasCaption());
   }
   else // other format
   {
@@ -1179,6 +1223,18 @@ void LatexDocVisitor::visitPost(DocMscFile *df)
   if (m_hide) return;
   endMscFile(df->hasCaption());
 }
+
+void LatexDocVisitor::visitPre(DocDiaFile *df)
+{
+  if (m_hide) return;
+  startDiaFile(df->file(),df->width(),df->height(),df->hasCaption());
+}
+
+void LatexDocVisitor::visitPost(DocDiaFile *df)
+{
+  if (m_hide) return;
+  endDiaFile(df->hasCaption());
+}
 void LatexDocVisitor::visitPre(DocLink *lnk)
 {
   if (m_hide) return;
@@ -1220,15 +1276,25 @@ void LatexDocVisitor::visitPost(DocRef *ref)
   }
 }
 
-void LatexDocVisitor::visitPre(DocSecRefItem *)
+void LatexDocVisitor::visitPre(DocSecRefItem *ref)
 {
   if (m_hide) return;
   m_t << "\\item \\contentsline{section}{";
+  static bool pdfHyperlinks = Config_getBool("PDF_HYPERLINKS");
+  if (pdfHyperlinks)
+  {
+    m_t << "\\hyperlink{" << ref->file() << "_" << ref->anchor() << "}{" ;
+  }
 }
 
 void LatexDocVisitor::visitPost(DocSecRefItem *ref) 
 {
   if (m_hide) return;
+  static bool pdfHyperlinks = Config_getBool("PDF_HYPERLINKS");
+  if (pdfHyperlinks)
+  {
+    m_t << "}";
+  }
   m_t << "}{\\ref{" << ref->file() << "_" << ref->anchor() << "}}{}" << endl;
 }
 
@@ -1416,6 +1482,7 @@ void LatexDocVisitor::visitPost(DocParamList *pl)
 void LatexDocVisitor::visitPre(DocXRefItem *x)
 {
   if (m_hide) return;
+  if (x->title().isEmpty()) return;
   m_t << "\\begin{DoxyRefDesc}{";
   filter(x->title());
   m_t << "}" << endl;
@@ -1435,9 +1502,10 @@ void LatexDocVisitor::visitPre(DocXRefItem *x)
   m_t << "}]";
 }
 
-void LatexDocVisitor::visitPost(DocXRefItem *)
+void LatexDocVisitor::visitPost(DocXRefItem *x)
 {
   if (m_hide) return;
+  if (x->title().isEmpty()) return;
   m_t << "\\end{DoxyRefDesc}" << endl;
 }
 
@@ -1472,13 +1540,33 @@ void LatexDocVisitor::visitPost(DocText *)
 void LatexDocVisitor::visitPre(DocHtmlBlockQuote *)
 {
   if (m_hide) return;
-  m_t << "\\begin{quotation}" << endl;
+  m_t << "\\begin{quote}" << endl;
 }
 
 void LatexDocVisitor::visitPost(DocHtmlBlockQuote *)
 {
   if (m_hide) return;
-  m_t << "\\end{quotation}" << endl;
+  m_t << "\\end{quote}" << endl;
+}
+
+void LatexDocVisitor::visitPre(DocVhdlFlow *)
+{
+  if (m_hide) return;
+}
+
+void LatexDocVisitor::visitPost(DocVhdlFlow *)
+{
+  if (m_hide) return;
+}
+
+void LatexDocVisitor::visitPre(DocParBlock *)
+{
+  if (m_hide) return;
+}
+
+void LatexDocVisitor::visitPost(DocParBlock *)
+{
+  if (m_hide) return;
 }
 
 void LatexDocVisitor::filter(const char *str)
@@ -1525,8 +1613,8 @@ void LatexDocVisitor::endLink(const QCString &ref,const QCString &file,const QCS
     m_t << "{"; 
     filter(theTranslator->trPageAbbreviation());
     m_t << "}{" << file;
-    if (!anchor.isEmpty()) m_t << "_" << anchor;
-    m_t << "}";
+    if (!file.isEmpty() && !anchor.isEmpty()) m_t << "_";
+    m_t << anchor << "}";
   }
 }
 
@@ -1562,49 +1650,14 @@ void LatexDocVisitor::startDotFile(const QCString &fileName,
   baseName.prepend("dot_");
   QCString outDir = Config_getString("LATEX_OUTPUT");
   QCString name = fileName;
-  writeDotGraphFromFile(name,outDir,baseName,EPS);
-  if (hasCaption)
-  {
-    m_t << "\n\\begin{DoxyImage}\n";
-  }
-  else
-  {
-    m_t << "\n\\begin{DoxyImageNoCaption}\n"
-           "  \\mbox{";
-  }
-  m_t << "\\includegraphics";
-  if (!width.isEmpty())
-  {
-    m_t << "[width=" << width << "]";
-  }
-  else if (!height.isEmpty())
-  {
-    m_t << "[height=" << height << "]";
-  }
-  else
-  {
-    m_t << "[width=\\textwidth]";
-  }
-  m_t << "{" << baseName << "}";
-
-  if (hasCaption)
-  {
-    m_t << "\n\\caption{";
-  }
+  writeDotGraphFromFile(name,outDir,baseName,GOF_EPS);
+  visitPreStart(m_t,hasCaption, baseName, width, height);
 }
 
 void LatexDocVisitor::endDotFile(bool hasCaption)
 {
   if (m_hide) return;
-  m_t << "}\n"; // end caption or mbox
-  if (hasCaption)
-  {
-    m_t << "\\end{DoxyImage}\n";
-  }
-  else
-  {
-    m_t << "\\end{DoxyImageNoCaption}\n";
-  }
+  visitPostEnd(m_t,hasCaption);
 }
 
 void LatexDocVisitor::startMscFile(const QCString &fileName,
@@ -1627,52 +1680,17 @@ void LatexDocVisitor::startMscFile(const QCString &fileName,
 
   QCString outDir = Config_getString("LATEX_OUTPUT");
   writeMscGraphFromFile(fileName,outDir,baseName,MSC_EPS); 
-  if (hasCaption)
-  {
-    m_t << "\n\\begin{DoxyImage}\n";
-  }
-  else
-  {
-    m_t << "\n\\begin{DoxyImageNoCaption}\n"
-           "  \\mbox{";
-  }
-  m_t << "\\includegraphics";
-  if (!width.isEmpty())
-  {
-    m_t << "[width=" << width << "]";
-  }
-  else if (!height.isEmpty())
-  {
-    m_t << "[height=" << height << "]";
-  }
-  else
-  {
-    m_t << "[width=\\textwidth]";
-  }
-  m_t << "{" << baseName << "}";
-
-  if (hasCaption)
-  {
-    m_t << "\n\\caption{";
-  }
+  visitPreStart(m_t,hasCaption, baseName, width, height);
 }
 
 void LatexDocVisitor::endMscFile(bool hasCaption)
 {
   if (m_hide) return;
-  m_t << "}\n"; // end caption or mbox
-  if (hasCaption)
-  {
-    m_t << "\\end{DoxyImage}\n";
-  }
-  else
-  {
-    m_t << "\\end{DoxyImageNoCaption}\n";
-  }
+  visitPostEnd(m_t,hasCaption);
 }
 
 
-void LatexDocVisitor::writeMscFile(const QCString &baseName)
+void LatexDocVisitor::writeMscFile(const QCString &baseName, DocVerbatim *s)
 {
   QCString shortName = baseName;
   int i;
@@ -1682,10 +1700,69 @@ void LatexDocVisitor::writeMscFile(const QCString &baseName)
   } 
   QCString outDir = Config_getString("LATEX_OUTPUT");
   writeMscGraphFromFile(baseName+".msc",outDir,shortName,MSC_EPS);
-  m_t << "\n\\begin{DoxyImageNoCaption}"
-         "  \\mbox{\\includegraphics";
-  m_t << "{" << shortName << "}";
-  m_t << "}\n"; // end mbox
-  m_t << "\\end{DoxyImageNoCaption}\n";
+  visitPreStart(m_t, s->hasCaption(), shortName, s->width(),s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
+}
+
+
+void LatexDocVisitor::startDiaFile(const QCString &fileName,
+                                   const QCString &width,
+                                   const QCString &height,
+                                   bool hasCaption
+                                  )
+{
+  QCString baseName=fileName;
+  int i;
+  if ((i=baseName.findRev('/'))!=-1)
+  {
+    baseName=baseName.right(baseName.length()-i-1);
+  }
+  if ((i=baseName.find('.'))!=-1)
+  {
+    baseName=baseName.left(i);
+  }
+  baseName.prepend("dia_");
+
+  QCString outDir = Config_getString("LATEX_OUTPUT");
+  writeDiaGraphFromFile(fileName,outDir,baseName,DIA_EPS);
+  visitPreStart(m_t,hasCaption, baseName, width, height);
+}
+
+void LatexDocVisitor::endDiaFile(bool hasCaption)
+{
+  if (m_hide) return;
+  visitPostEnd(m_t,hasCaption);
+}
+
+
+void LatexDocVisitor::writeDiaFile(const QCString &baseName, DocVerbatim *s)
+{
+  QCString shortName = baseName;
+  int i;
+  if ((i=shortName.findRev('/'))!=-1)
+  {
+    shortName=shortName.right(shortName.length()-i-1);
+  }
+  QCString outDir = Config_getString("LATEX_OUTPUT");
+  writeDiaGraphFromFile(baseName+".dia",outDir,shortName,DIA_EPS);
+  visitPreStart(m_t, s->hasCaption(), shortName, s->width(), s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
+}
+
+void LatexDocVisitor::writePlantUMLFile(const QCString &baseName, DocVerbatim *s)
+{
+  QCString shortName = baseName;
+  int i;
+  if ((i=shortName.findRev('/'))!=-1)
+  {
+    shortName=shortName.right(shortName.length()-i-1);
+  }
+  QCString outDir = Config_getString("LATEX_OUTPUT");
+  generatePlantUMLOutput(baseName,outDir,PUML_EPS);
+  visitPreStart(m_t, s->hasCaption(), shortName, s->width(), s->height());
+  visitCaption(this, s->children());
+  visitPostEnd(m_t, s->hasCaption());
 }
 
