@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: membergroup.h,v 1.13 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,24 +18,25 @@
 #ifndef MEMBERGROUP_H
 #define MEMBERGROUP_H
 
-#include "qtbc.h"
 #include <qlist.h>
-#include <qfile.h>
 #include "sortdict.h"
+#include "types.h"
 
 #define DOX_NOGROUP -1
 
+class MemberList;
 class MemberDef;
 class ClassDef;
 class NamespaceDef;
 class FileDef;
-class MemberList;
 class GroupDef;
 class OutputList;
 class Definition;
 class StorageIntf;
+class FTextStream;
 struct ListItemInfo;
 
+/** A class representing a group of members. */
 class MemberGroup 
 {
   public:
@@ -46,21 +47,28 @@ class MemberGroup
     QCString header() const { return grpHeader; }
     int groupId() const { return grpId; }
     void insertMember(MemberDef *md);
-    void setAnchors(ClassDef *);
+    void setAnchors();
     void writePlainDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+               ClassDef *inheritedFrom,const char *inheritId);
     void writeDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
-
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+               bool showInline=FALSE);
     void writeDocumentation(OutputList &ol,const char *scopeName,
-               Definition *container);
+               Definition *container,bool showEnumValues,bool showInline);
     void writeDocumentationPage(OutputList &ol,const char *scopeName,
                Definition *container);
-    QCString documentation() { return doc; }
-    bool allMembersInSameSection() { return inSameSection; }
+    void writeTagFile(FTextStream &);
+    void addGroupedInheritedMembers(OutputList &ol,ClassDef *cd,
+               MemberListType lt,
+               ClassDef *inheritedFrom,const QCString &inheritId);
+
+    const QCString &documentation() const { return doc; }
+    bool allMembersInSameSection() const { return inSameSection; }
     void addToDeclarationSection();
-    int countDecMembers();
+    int countDecMembers(GroupDef *gd=0);
     int countDocMembers();
+    int countGroupedInheritedMembers(MemberListType lt);
     void distributeMemberGroupDocumentation();
     void findSectionsInDocumentation();
     int varCount() const;
@@ -73,6 +81,7 @@ class MemberGroup
     int friendCount() const;
     int numDecMembers() const;
     int numDocMembers() const;
+    int countInheritableMembers(ClassDef *inheritedFrom) const;
     void setInGroup(bool b);
     void addListReferences(Definition *d);
     void setRefItems(const QList<ListItemInfo> *sli);
@@ -99,10 +108,12 @@ class MemberGroup
     QList<ListItemInfo> *m_xrefListItems;
 };
 
+/** A list of MemberGroup objects. */
 class MemberGroupList : public QList<MemberGroup>
 {
 };
 
+/** An iterator for MemberGroup objects in a MemberGroupList. */
 class MemberGroupListIterator : public QListIterator<MemberGroup>
 {
   public:
@@ -110,18 +121,20 @@ class MemberGroupListIterator : public QListIterator<MemberGroup>
       QListIterator<MemberGroup>(l) {}
 };
 
+/** A sorted dictionary of MemberGroup objects. */
 class MemberGroupSDict : public SIntDict<MemberGroup>
 {
   public:
     MemberGroupSDict(int size=17) : SIntDict<MemberGroup>(size) {}
    ~MemberGroupSDict() {}
-    int compareItems(GCI item1,GCI item2)
+ private:
+    int compareValues(const MemberGroup *item1,const MemberGroup *item2) const
     {
-      return ((MemberGroup *)item1)->groupId() - ((MemberGroup*)item2)->groupId();
+      return item1->groupId() - item2->groupId();
     }
 };
 
-
+/** Data collected for a member group */
 struct MemberGroupInfo
 {
   MemberGroupInfo() : m_sli(0) {}
@@ -134,19 +147,4 @@ struct MemberGroupInfo
   QList<ListItemInfo> *m_sli;
 };
 
-//class MemberGroupDict : public QIntDict<MemberGroup>
-//{
-//  public:
-//    MemberGroupDict(int size) : QIntDict<MemberGroup>(size) {}
-//   ~MemberGroupDict() {}
-//};
-
-//class MemberGroupDictIterator : public QIntDictIterator<MemberGroup>
-//{
-//  public:
-//    MemberGroupDictIterator(const MemberGroupDict &d) : 
-//      QIntDictIterator<MemberGroup>(d) {}
-//   ~MemberGroupDictIterator() {}
-//};
-
 #endif
