@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,11 +36,10 @@ class MemberList : public QList<MemberDef>
     MemberList(MemberListType lt);
    ~MemberList();
     MemberListType listType() const { return m_listType; }
-    QCString listTypeAsString(MemberListType type) const;
+    static QCString listTypeAsString(MemberListType type);
     bool insert(uint index,const MemberDef *md);
     void inSort(const MemberDef *md);
     void append(const MemberDef *md);
-    int compareItems(QCollection::Item item1,QCollection::Item item2);
     int varCount() const       { ASSERT(m_numDecMembers!=-1); return m_varCnt;     }
     int funcCount() const      { ASSERT(m_numDecMembers!=-1); return m_funcCnt;    }
     int enumCount() const      { ASSERT(m_numDecMembers!=-1); return m_enumCnt;    }
@@ -56,12 +55,11 @@ class MemberList : public QList<MemberDef>
     void countDocMembers(bool countEnumValues=FALSE);
     int countInheritableMembers(ClassDef *inheritedFrom) const;
     void writePlainDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,
-               GroupDef *gd,const DefinitionIntf::DefType compoundType,
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd, GroupDef *gd,
                ClassDef *inheritedFrom,const char *inheritId);
     void writeDeclarations(OutputList &ol,
                ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
-               const char *title,const char *subtitle,const DefinitionIntf::DefType compoundType,
+               const char *title,const char *subtitle,
                bool showEnumValues=FALSE,bool showInline=FALSE,
                ClassDef *inheritedFrom=0,MemberListType lt=MemberListType_pubMethods);
     void writeDocumentation(OutputList &ol,const char *scopeName,
@@ -69,6 +67,7 @@ class MemberList : public QList<MemberDef>
     void writeSimpleDocumentation(OutputList &ol,Definition *container);
     void writeDocumentationPage(OutputList &ol,
                const char *scopeName, Definition *container);
+    void writeTagFile(FTextStream &);
     bool declVisible() const;
     void addMemberGroup(MemberGroup *mg);
     void setInGroup(bool inGroup) { m_inGroup=inGroup; }
@@ -82,6 +81,7 @@ class MemberList : public QList<MemberDef>
     void unmarshal(StorageIntf *s);
 
   private:
+    int compareValues(const MemberDef *item1,const MemberDef *item2) const;
     int m_varCnt;
     int m_funcCnt;
     int m_enumCnt;
@@ -121,7 +121,8 @@ class MemberSDict : public SDict<MemberDef>
   public:
     MemberSDict(int size=17) : SDict<MemberDef>(size) {}
     virtual ~MemberSDict() {}
-    int compareItems(QCollection::Item item1,QCollection::Item item2);
+  private:
+    int compareValues(const MemberDef *item1,const MemberDef *item2) const;
 };
 
 
