@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * 
  *
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -19,7 +19,9 @@
 #ifndef _PRINTDOCVISITOR_H
 #define _PRINTDOCVISITOR_H
 
+#include <qglobal.h>
 #include "docvisitor.h"
+#include "htmlentity.h"
 
 /*! Concrete visitor implementation for pretty printing */
 class PrintDocVisitor : public DocVisitor
@@ -55,38 +57,14 @@ class PrintDocVisitor : public DocVisitor
     void visit(DocSymbol *s)
     {
       indent_leaf();
-      switch(s->symbol())
+      const char *res = HtmlEntityMapper::instance()->utf8(s->symbol(),TRUE);
+      if (res)
       {
-	case DocSymbol::BSlash:  printf("\\"); break;
-	case DocSymbol::At:      printf("@"); break;
-	case DocSymbol::Less:    printf("<"); break;
-	case DocSymbol::Greater: printf(">"); break;
-	case DocSymbol::Amp:     printf("&"); break;
-	case DocSymbol::Dollar:  printf("$"); break;
-	case DocSymbol::Hash:    printf("#"); break;
-	case DocSymbol::Percent: printf("%%"); break;
-	case DocSymbol::Copy:    printf("&copy;"); break;
-	case DocSymbol::Apos:    printf("'"); break;
-	case DocSymbol::Quot:    printf("\""); break;
-        case DocSymbol::Lsquo:   printf("&lsquo;"); break;
-        case DocSymbol::Rsquo:   printf("&rsquo;"); break;
-        case DocSymbol::Ldquo:   printf("&ldquo;"); break;
-        case DocSymbol::Rdquo:   printf("&rdquo;"); break;
-        case DocSymbol::Ndash:   printf("&ndash;"); break;
-        case DocSymbol::Mdash:   printf("&mdash;"); break;
-	case DocSymbol::Uml:     printf("&%cuml;",s->letter()); break;
-	case DocSymbol::Acute:   printf("&%cacute;",s->letter()); break;
-	case DocSymbol::Grave:   printf("&%cgrave;",s->letter()); break;
-	case DocSymbol::Circ:    printf("&%ccirc;",s->letter()); break;
-	case DocSymbol::Tilde:   printf("&%ctilde;",s->letter()); break;
-	case DocSymbol::Szlig:   printf("&szlig;"); break;
-	case DocSymbol::Cedil:   printf("&%ccedul;",s->letter()); break;
-	case DocSymbol::Ring:    printf("&%cring;",s->letter()); break;
-	case DocSymbol::Nbsp:    printf("&nbsp;"); break;
-	case DocSymbol::Aelig:   printf("&aelig;"); break;
-	case DocSymbol::AElig:   printf("&AElig;"); break;
-	default:
-	  printf("Error: unknown symbol found\n");
+        printf("%s",res);
+      }
+      else
+      {
+        printf("print: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance()->html(s->symbol(),TRUE));
       }
     }
     void visit(DocURL *u)
@@ -110,35 +88,35 @@ class PrintDocVisitor : public DocVisitor
       switch (s->style())
       {
         case DocStyleChange::Bold:
-         if (s->enable()) printf("<bold>"); else printf("</bold>");
-         break;
+          if (s->enable()) printf("<bold>"); else printf("</bold>");
+          break;
         case DocStyleChange::Italic:
-         if (s->enable()) printf("<italic>"); else printf("</italic>");
-         break;
+          if (s->enable()) printf("<italic>"); else printf("</italic>");
+          break;
         case DocStyleChange::Code:
-         if (s->enable()) printf("<code>"); else printf("</code>");
-         break;
+          if (s->enable()) printf("<code>"); else printf("</code>");
+          break;
         case DocStyleChange::Subscript:
-         if (s->enable()) printf("<sub>"); else printf("</sub>");
-         break;
+          if (s->enable()) printf("<sub>"); else printf("</sub>");
+          break;
         case DocStyleChange::Superscript:
-         if (s->enable()) printf("<sup>"); else printf("</sup>");
-         break;
+          if (s->enable()) printf("<sup>"); else printf("</sup>");
+          break;
         case DocStyleChange::Center:
-         if (s->enable()) printf("<center>"); else printf("</center>");
-         break;
+          if (s->enable()) printf("<center>"); else printf("</center>");
+          break;
         case DocStyleChange::Small:
-         if (s->enable()) printf("<small>"); else printf("</small>");
-         break;
+          if (s->enable()) printf("<small>"); else printf("</small>");
+          break;
         case DocStyleChange::Preformatted:
-         if (s->enable()) printf("<pre>"); else printf("</pre>");
-         break;
+          if (s->enable()) printf("<pre>"); else printf("</pre>");
+          break;
         case DocStyleChange::Div:
-         if (s->enable()) printf("<div>"); else printf("</div>");
-         break;
+          if (s->enable()) printf("<div>"); else printf("</div>");
+          break;
         case DocStyleChange::Span:
-         if (s->enable()) printf("<span>"); else printf("</span>");
-         break;
+          if (s->enable()) printf("<span>"); else printf("</span>");
+          break;
       }
     }
     void visit(DocVerbatim *s)
@@ -153,8 +131,10 @@ class PrintDocVisitor : public DocVisitor
         case DocVerbatim::ManOnly: printf("<manonly>"); break;
         case DocVerbatim::LatexOnly: printf("<latexonly>"); break;
         case DocVerbatim::XmlOnly: printf("<xmlonly>"); break;
+        case DocVerbatim::DocbookOnly: printf("<docbookonly>"); break;
         case DocVerbatim::Dot: printf("<dot>"); break;
         case DocVerbatim::Msc: printf("<msc>"); break;
+        case DocVerbatim::PlantUML: printf("<plantuml>"); break;
       }
       printf("%s",s->text().data());
       switch(s->type())
@@ -166,8 +146,10 @@ class PrintDocVisitor : public DocVisitor
         case DocVerbatim::ManOnly: printf("</manonly>"); break;
         case DocVerbatim::LatexOnly: printf("</latexonly>"); break;
         case DocVerbatim::XmlOnly: printf("</xmlonly>"); break;
+        case DocVerbatim::DocbookOnly: printf("</docbookonly>"); break;
         case DocVerbatim::Dot: printf("</dot>"); break;
         case DocVerbatim::Msc: printf("</msc>"); break;
+        case DocVerbatim::PlantUML: printf("</plantuml>"); break;
       }
     }
     void visit(DocAnchor *a)
@@ -185,6 +167,7 @@ class PrintDocVisitor : public DocVisitor
         case DocInclude::IncWithLines: printf("incwithlines"); break;
         case DocInclude::DontInclude: printf("dontinclude"); break;
         case DocInclude::HtmlInclude: printf("htmlinclude"); break;
+        case DocInclude::LatexInclude: printf("latexinclude"); break;
         case DocInclude::VerbInclude: printf("verbinclude"); break;
         case DocInclude::Snippet: printf("snippet"); break;
       }
@@ -495,11 +478,12 @@ class PrintDocVisitor : public DocVisitor
       printf("<image src=\"%s\" type=\"",img->name().data());
       switch(img->type())
       {
-        case DocImage::Html: printf("html"); break;
-        case DocImage::Latex: printf("latex"); break;
-        case DocImage::Rtf: printf("rtf"); break;
+        case DocImage::Html:    printf("html"); break;
+        case DocImage::Latex:   printf("latex"); break;
+        case DocImage::Rtf:     printf("rtf"); break;
+        case DocImage::DocBook: printf("docbook"); break;
       }
-      printf("\" width=%s height=%s>\n",img->width().data(),img->height().data());
+      printf("\" %s %s>\n",img->width().data(),img->height().data());
     }
     void visitPost(DocImage *) 
     {
@@ -526,6 +510,16 @@ class PrintDocVisitor : public DocVisitor
       indent_post();
       printf("</mscfile>\n");
     }
+    void visitPre(DocDiaFile *df)
+    {
+      indent_pre();
+      printf("<diafile src=\"%s\">\n",df->name().data());
+    }
+    void visitPost(DocDiaFile *)
+    {
+      indent_post();
+      printf("</diafile>\n");
+    }
     void visitPre(DocLink *lnk)
     {
       indent_pre();
@@ -680,6 +674,26 @@ class PrintDocVisitor : public DocVisitor
       indent_post();
       printf("</blockquote>\n");
     }
+    void visitPre(DocVhdlFlow *)
+    {
+      indent_pre();
+      printf("<vhdlflow>\n");
+    }
+    void visitPost(DocVhdlFlow *)
+    {
+      indent_post();
+      printf("</vhdlflow>\n");
+    }
+    void visitPre(DocParBlock *)
+    {
+      indent_pre();
+      printf("<parblock>\n");
+    }
+    void visitPost(DocParBlock *)
+    {
+      indent_post();
+      printf("</parblock>\n");
+    }
 
   private:
     // helper functions
