@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: tagreader.h,v 1.2 2001/03/19 19:27:41 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -19,10 +19,8 @@
 #ifndef TAGREADER_H
 #define TAGREADER_H
 
-#include "qtbc.h"
-
 class Entry;
 
-void parseTagFile(Entry *root,const char *fullPathName,const char *fileName);
+void parseTagFile(Entry *root,const char *fullPathName);
 
 #endif
