@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2013 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -11,7 +11,7 @@
  * input used in their production; they are not affected by this license.
  *
  * Portuguese translation version 20110428
- *    Maintainer (from 04/28/2011): 
+ *    Maintainer (from 04/28/2011):
  *       Fabio "FJTC" Jun Takada Chino <jun-chino at uol.com.br>
  *    Maintainer (until 04/28/2011):
  *       Rui Godinho Lopes <rui at ruilopes.com>
@@ -24,11 +24,14 @@
  * VERSION HISTORY
  * ---------------
  * History:
+ * 20131129:
+ *  - Updated to 1.8.5;
+ *  - Translation in the method trFileMembers() fixed;
  * 20110628:
- *	- Updated to 1.7.5; 
+ *	- Updated to 1.7.5;
  *      - All obsolete methods have been removed;
  * 20110428
- *   - Updated to doxygen 1.6.3 using the Brazilian Portuguese as the base. 
+ *   - Updated to doxygen 1.6.3 using the Brazilian Portuguese as the base.
  *     Requires revision by a Portuguese (Portugal native speaker);
  * 007 09 june 2003
  *   ! Updated for doxygen v1.3.1
@@ -52,24 +55,24 @@
 #define TRANSLATOR_PT_H
 
 
-class TranslatorPortuguese : public TranslatorAdapter_1_8_0
+class TranslatorPortuguese : public Translator
 {
   public:
 
     // --- Language control methods -------------------
 
-    /*! Used for identification of the language. The identification 
-     * should not be translated. It should be replaced by the name 
+    /*! Used for identification of the language. The identification
+     * should not be translated. It should be replaced by the name
      * of the language in English using lower-case characters only
-     * (e.g. "czech", "japanese", "russian", etc.). It should be equal to 
+     * (e.g. "czech", "japanese", "russian", etc.). It should be equal to
      * the identification used in language.cpp.
      */
     QCString idLanguage()
     { return "portuguese"; }
 
-    /*! Used to get the LaTeX command(s) for the language support. 
+    /*! Used to get the LaTeX command(s) for the language support.
      *  This method should return string with commands that switch
-     *  LaTeX to the desired language.  For example 
+     *  LaTeX to the desired language.  For example
      *  <pre>"\\usepackage[german]{babel}\n"
      *  </pre>
      *  or
@@ -77,17 +80,13 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
      *  "\\usepackage[latin2]{inputenc}\n"
      *  "\\usepackage[T1]{fontenc}\n"
      *  </pre>
-     * 
+     *
      * The English LaTeX does not use such commands.  Because of this
      * the empty string is returned in this implementation.
      */
     virtual QCString latexLanguageSupportCommand()
     { return "Portuguese"; }
 
-    /*! return the language charset. This will be used for the HTML output */
-    virtual QCString idLanguageCharset()
-    { return "utf-8"; }
-
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
@@ -119,7 +118,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
-        return "Documentação dos campos e atributos"; 
+        return "Documentação dos campos e atributos";
       }
       else
       {
@@ -220,7 +219,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       }
       else
       {
-        return "Ficheiros membro";
+        return "Membros dos Ficheiros";
       }
     }
 
@@ -251,7 +250,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
 
     /*! This is an introduction to the annotated compound list. */
     QCString trCompoundListDescription()
-    {       
+    {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
       {
         return "Lista das estruturas de dados com uma breve descrição:";
@@ -279,7 +278,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
         result+=" documentadas";
       }
       result+=" com referência para ";
-      if (!extractAll) 
+      if (!extractAll)
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
@@ -290,7 +289,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
           result+="a documentação de cada membro:";
         }
       }
-      else 
+      else
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
@@ -319,9 +318,9 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
         if (!extractAll) result+="documentados ";
       }
       result+="com referência para ";
-      if (extractAll) 
+      if (extractAll)
         result+="o ficheiro a que pertecem:";
-      else 
+      else
         result+="a respectiva documentação:";
       return result;
     }
@@ -363,7 +362,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     QCString trCompoundIndex()
     {
       if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
-      { 
+      {
         return "Índice das estruturas de dados";
       }
       else
@@ -418,12 +417,6 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     QCString trDefines()
     { return "Macros"; }
 
-    /*! This is used in the documentation of a file as a header before the
-     *  list of function prototypes
-     */
-    QCString trFuncProtos()
-    { return "Protótipos de funções"; }
-
     /*! This is used in the documentation of a file as a header before the
      *  list of typedefs
      */
@@ -452,7 +445,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
      *  list of (global) variables
      */
     QCString trEnumerationValues()
-    { return "Valores da enumeração"; }
+    { return "Valores de enumerações"; }
 
     /*! This is used in the documentation of a file before the list of
      *  documentation blocks for defines
@@ -460,12 +453,6 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     QCString trDefineDocumentation()
     { return "Documentação das macros"; }
 
-    /*! This is used in the documentation of a file/namespace before the list
-     *  of documentation blocks for function prototypes
-     */
-    QCString trFunctionPrototypeDocumentation()
-    { return "Documentação dos protótipos de funções"; }
-
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for typedefs
      */
@@ -515,12 +502,6 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       result+=(QCString)" por";
       return result;
     }
-    /*! This is part of the sentence used in the standard footer of each page.
-     */
-    QCString trWrittenBy()
-    {
-      return "escrito por";
-    }
 
     /*! this text is put before a class diagram */
     QCString trClassDiagram(const char *clName)
@@ -616,6 +597,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
         case ClassDef::Protocol:   result+="ao protocolo "; break;
         case ClassDef::Category:   result+="à categoria "; break;
         case ClassDef::Exception:  result+="à excepção "; break;
+        default: break;
       }
       if (isTemplate) result+="Template ";
       result+=(QCString)clName;
@@ -778,6 +760,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
         case ClassDef::Protocol:   result+="este protocolo"; break;
         case ClassDef::Category:   result+="esta categoria"; break;
         case ClassDef::Exception:  result+="esta excepção"; break;
+        default: break;
       }
       result+=" foi gerada a partir ";
       if (single) result+=" do seguinte ficheiro:";
@@ -785,15 +768,6 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       return result;
     }
 
-    /*! This is in the (quick) index as a link to the alphabetical compound
-     * list.
-     */
-    virtual QCString trAlphabeticalList()
-    {
-      return "Lista alfabética";
-    }
-
-
 //////////////////////////////////////////////////////////////////////////
 // new since 0.49-990901
 //////////////////////////////////////////////////////////////////////////
@@ -1086,16 +1060,6 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       return "Lista de teste";
     }
 
-//////////////////////////////////////////////////////////////////////////
-// new since 1.2.1
-//////////////////////////////////////////////////////////////////////////
-
-    /*! Used as a section header for KDE-2 IDL methods */
-    virtual QCString trDCOPMethods()
-    {
-      return "Métodos DCOP";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.2.2
 //////////////////////////////////////////////////////////////////////////
@@ -1322,7 +1286,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
 // new since 1.2.13
 //////////////////////////////////////////////////////////////////////////
 
-    /*! used in member documentation blocks to produce a list of 
+    /*! used in member documentation blocks to produce a list of
      *  members that are implemented by this one.
      */
     virtual QCString trImplementedFromList(int numEntries)
@@ -1366,7 +1330,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
 // new since 1.2.18
 //////////////////////////////////////////////////////////////////////////
 
-    /*! Used as a header for declaration section of the events found in 
+    /*! Used as a header for declaration section of the events found in
      * a C# program
      */
     virtual QCString trEvents()
@@ -1386,43 +1350,43 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     /*! Used as a heading for a list of Java class types with package scope.
      */
     virtual QCString trPackageTypes()
-    { 
+    {
       return "Tipos do Pacote";
     }
-    /*! Used as a heading for a list of Java class functions with package 
-     * scope. 
+    /*! Used as a heading for a list of Java class functions with package
+     * scope.
      */
     virtual QCString trPackageMembers()
-    { 
+    {
       return "Funções do Pacote";
     }
-    /*! Used as a heading for a list of static Java class functions with 
+    /*! Used as a heading for a list of static Java class functions with
      *  package scope.
      */
     virtual QCString trStaticPackageMembers()
-    { 
+    {
       return "Funções Estáticas do Pacote";
     }
-    /*! Used as a heading for a list of Java class variables with package 
+    /*! Used as a heading for a list of Java class variables with package
      * scope.
      */
     virtual QCString trPackageAttribs()
-    { 
+    {
       return "Atributos do Pacote";
     }
-    /*! Used as a heading for a list of static Java class variables with 
+    /*! Used as a heading for a list of static Java class variables with
      * package scope.
      */
     virtual QCString trStaticPackageAttribs()
-    { 
+    {
       return "Atributos Estáticos do Pacote";
     }
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.3.1
 //////////////////////////////////////////////////////////////////////////
 
-    /*! Used in the quick index of a class/file/namespace member list page 
+    /*! Used in the quick index of a class/file/namespace member list page
      *  to link to the unfiltered list of all members.
      */
     virtual QCString trAll()
@@ -1434,19 +1398,11 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     {
       return "Grafo de chamadas desta função:";
     }
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.3.3 - Based on the Brazilian Portuguese Translation
 //////////////////////////////////////////////////////////////////////////
 
-    /*! When the search engine is enabled this text is put in the header
-     *  of each page before the field where one can enter the text to search
-     *  for.
-     */
-    virtual QCString trSearchForIndex()
-    {
-      return "Procurar por";
-    }
     /*! This string is used as the title for the page listing the search
      *  results.
      */
@@ -1485,7 +1441,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     {
       return "Resultados:";
     }
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.3.8
 //////////////////////////////////////////////////////////////////////////
@@ -1519,7 +1475,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     virtual QCString trDirectories()
     { return "Diretórios"; }
 
-    /*! This returns a sentences that introduces the directory hierarchy. 
+    /*! This returns a sentences that introduces the directory hierarchy.
      *  and the fact that it is sorted alphabetically per level
      */
     virtual QCString trDirDescription()
@@ -1539,10 +1495,10 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
      *  (\a first_capital) and in sigular or plural form (\a singular).
      */
     virtual QCString trDir(bool first_capital, bool singular)
-    { 
+    {
       QCString result((first_capital ? "Diretório" : "diretório"));
       if (!singular) result+="s";
-      return result; 
+      return result;
     }
 //////////////////////////////////////////////////////////////////////////
 // new since 1.4.1
@@ -1556,7 +1512,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
        return "Este é um método provido por conveniência. "
        		"Ele difere do método acima apenas na lista de "
        		"argumentos que devem ser utilizados.";
-    }    
+    }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.4.6
@@ -1568,22 +1524,22 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       return "Este é o diagrama das funções que utilizam esta função:";
     }
 
-    /*! This is used in the documentation of a file/namespace before the list 
+    /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for enumeration values
      */
     virtual QCString trEnumerationValueDocumentation()
     { return "Documentação da enumeração"; }
 
-  
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.5.4 (mainly for Fortran)
 //////////////////////////////////////////////////////////////////////////
-    
+
     /*! header that is put before the list of member subprograms (Fortran). */
     virtual QCString trMemberFunctionDocumentationFortran()
     { return "Documentação de Função Membro/Subrotina"; }
 
-    /*! This is put above each page as a link to the list of annotated data types (Fortran). */    
+    /*! This is put above each page as a link to the list of annotated data types (Fortran). */
     virtual QCString trCompoundListFortran()
     { return "Lista de Tipos de Dados"; }
 
@@ -1601,18 +1557,18 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       QCString result="Esta é a lista de todos os membros ";
       if (!extractAll) result+="documentados ";
       result+="dos tipos de dados com links para ";
-      if (extractAll) 
+      if (extractAll)
       {
         result+="a documentação dos tipos de dados para cada membro:";
       }
-      else 
+      else
       {
         result+="os tipos de dados a que pertencem:";
       }
       return result;
     }
 
-    /*! This is used in LaTeX as the title of the chapter with the 
+    /*! This is used in LaTeX as the title of the chapter with the
      * annotated compound index (Fortran).
      */
     virtual QCString trCompoundIndexFortran()
@@ -1624,24 +1580,24 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
     virtual QCString trTypeDocumentation()
     { return "Documentação dos Tipos de Dados"; }
 
-    /*! This is used in the documentation of a file as a header before the 
+    /*! This is used in the documentation of a file as a header before the
      *  list of (global) subprograms (Fortran).
      */
     virtual QCString trSubprograms()
     { return "Funções/Subrotinas"; }
 
-    /*! This is used in the documentation of a file/namespace before the list 
+    /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for subprograms (Fortran)
      */
     virtual QCString trSubprogramDocumentation()
     { return "Documentação da Função/Subrotina"; }
 
-    /*! This is used in the documentation of a file/namespace/group before 
+    /*! This is used in the documentation of a file/namespace/group before
      *  the list of links to documented compounds (Fortran)
      */
      virtual QCString trDataTypes()
     { return "Tipos de Dados"; }
-    
+
     /*! used as the title of page containing all the index of all modules (Fortran). */
     virtual QCString trModulesList()
     { return "Lista de Módulos"; }
@@ -1663,7 +1619,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       QCString result = (QCString)"Referência ";
 
       if (isTemplate) result+="da Template ";
-      
+
       switch(compType)
       {
         case ClassDef::Class:      result+="do Modulo "; break;
@@ -1673,55 +1629,56 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
         case ClassDef::Protocol:   result+="do Protocolo "; break;
         case ClassDef::Category:   result+="da Categoria "; break;
         case ClassDef::Exception:  result+="da Exceção "; break;
+        default: break;
       }
-      result += clName; 
-     
+      result += clName;
+
       return result;
     }
     /*! used as the title of the HTML page of a module (Fortran) */
     virtual QCString trModuleReference(const char *namespaceName)
     {
       QCString result = "Referência do Módulo ";
-      result += namespaceName;        
+      result += namespaceName;
       return result;
     }
-    
+
     /*! This is put above each page as a link to all members of modules. (Fortran) */
     virtual QCString trModulesMembers()
     { return "Membros do Módulo"; }
 
     /*! This is an introduction to the page with all modules members (Fortran) */
     virtual QCString trModulesMemberDescription(bool extractAll)
-    { 
+    {
       QCString result="Esta é a lista de todos os membros ";
       if (!extractAll) result+="documentados ";
       result+="dos módulos com links para ";
-      if (extractAll) 
+      if (extractAll)
       {
         result+="a documentação dos módulos para cada membro:";
       }
-      else 
+      else
       {
         result+="os módulos a que pertencem:";
       }
       return result;
     }
 
-    /*! This is used in LaTeX as the title of the chapter with the 
+    /*! This is used in LaTeX as the title of the chapter with the
      *  index of all modules (Fortran).
      */
     virtual QCString trModulesIndex()
     { return "Índice dos Módulos"; }
-    
+
     /*! This is used for translation of the word that will possibly
-     *  be followed by a single name or by a list of names 
+     *  be followed by a single name or by a list of names
      *  of the category.
      */
     virtual QCString trModule(bool first_capital, bool singular)
-    {       
+    {
       QCString result((first_capital ? "Modulo" : "modulo"));
       if (!singular)  result+="s";
-      return result; 
+      return result;
     }
     /*! This is put at the bottom of a module documentation page and is
      *  followed by a list of files that were used to generate the page.
@@ -1740,42 +1697,43 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
         case ClassDef::Protocol:   result+="esto protocolo "; break;
         case ClassDef::Category:   result+="esta categoria "; break;
         case ClassDef::Exception:  result+="esta exceção "; break;
+        default: break;
       }
-      
+
       result+=" foi gerada a partir do";
-      if (single) 
+      if (single)
     	result+=" seguinte ficheiro:";
       else
     	result+="s seguintes ficheiros:";
-      
+
       return result;
     }
     /*! This is used for translation of the word that will possibly
-     *  be followed by a single name or by a list of names 
+     *  be followed by a single name or by a list of names
      *  of the category.
      */
     virtual QCString trType(bool first_capital, bool singular)
-    { 
+    {
       QCString result((first_capital ? "Tipo" : "tipo"));
       if (!singular)  result+="s";
-      return result; 
+      return result;
     }
     /*! This is used for translation of the word that will possibly
-     *  be followed by a single name or by a list of names 
+     *  be followed by a single name or by a list of names
      *  of the category.
      */
     virtual QCString trSubprogram(bool first_capital, bool singular)
-    { 
+    {
       QCString result((first_capital ? "Subprograma" : "subprograma"));
       if (!singular)  result+="s";
-      return result; 
+      return result;
     }
 
     /*! C# Type Contraint list */
     virtual QCString trTypeConstraints()
     {
       return "Restrições do Tipo";
-    }    
+    }
 
 //////////////////////////////////////////////////////////////////////////
 // new since 1.6.0 (mainly for the new search engine)
@@ -1816,12 +1774,6 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
 // new since 1.6.3 (missing items for the directory pages)
 //////////////////////////////////////////////////////////////////////////
 
-    /*! introduction text for the directory dependency graph */
-    virtual QCString trDirDependency(const char *name)
-    {
-      return (QCString)"Grafo de dependência de diretórios para " + name;
-    }
-
     /*! when clicking a directory dependency label, a page with a
      *  table is shown. The heading for the first column mentions the
      *  source file that has a relation to another file.
@@ -1840,7 +1792,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       return (QCString)"Inclui ficheiro em "+name;
     }
 
-    /** Compiles a date string. 
+    /** Compiles a date string.
      *  @param year Year in 4 digits
      *  @param month Month of the year: 1=January
      *  @param day Day of the Month: 1..31
@@ -1866,7 +1818,7 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
       }
       return sdate;
     }
-    
+
 //////////////////////////////////////////////////////////////////////////
 // new since 1.7.5
 //////////////////////////////////////////////////////////////////////////
@@ -1885,7 +1837,186 @@ class TranslatorPortuguese : public TranslatorAdapter_1_8_0
 
     /*! Header for the graph showing the directory dependencies */
     virtual QCString trDirDepGraph(const char *name)
-    { return QCString("Grafo de dependências do directório ")+name+":"; }       
+    { return QCString("Grafo de dependências do directório ")+name+":"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.0
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Detail level selector shown for hierarchical indices */
+    virtual QCString trDetailLevel()
+    { return "nível de detalhes"; }
+
+    /*! Section header for list of template parameters */
+    virtual QCString trTemplateParameters()
+    { return "Parâmetros de template"; }
+
+    /*! Used in dot graph when UML_LOOK is enabled and there are many fields */
+    virtual QCString trAndMore(const QCString &number)
+    { return "e "+number+" mais..."; }
+
+    /*! Used file list for a Java enum */
+    virtual QCString trEnumGeneratedFromFiles(bool single)
+    { QCString result = "A documentação para esta enumeração foi gerada a partir";
+      if (single) {
+        result += "do seguinte ficheiro:";
+      } else {
+        result += "dos seguintes ficheiros:";
+      }
+      return result;
+    }
+
+    /*! Header of a Java enum page (Java enums are represented as classes). */
+    virtual QCString trEnumReference(const char *name)
+    {
+      QCString result = "Referência da enumeração ";
+      return result + name;
+    }
+
+    /*! Used for a section containing inherited members */
+    virtual QCString trInheritedFrom(const char *members,const char *what)
+    { return QCString(members)+" herdados de "+what; }
+
+    /*! Header of the sections with inherited members specific for the
+     *  base class(es)
+     */
+    virtual QCString trAdditionalInheritedMembers()
+    { return "Outros membros herdados"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.2
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a tooltip for the toggle button that appears in the
+     *  navigation tree in the HTML output when GENERATE_TREEVIEW is
+     *  enabled. This tooltip explains the meaning of the button.
+     */
+    virtual QCString trPanelSynchronisationTooltip(bool enable)
+    {
+      QCString opt = enable ? "ativar" : "desativar";
+      return "clique para "+opt+" a sincronização do painel";
+    }
+
+    /*! Used in a method of an Objective-C class that is declared in a
+     *  a category. Note that the @1 marker is required and is replaced
+     *  by a link.
+     */
+    virtual QCString trProvidedByCategory()
+    {
+      return "Provido pela categoria @1.";
+    }
+
+    /*! Used in a method of an Objective-C category that extends a class.
+     *  Note that the @1 marker is required and is replaced by a link to
+     *  the class method.
+     */
+    virtual QCString trExtendsClass()
+    {
+      return "estende a classe @1.";
+    }
+
+    /*! Used as the header of a list of class methods in Objective-C.
+     *  These are similar to static public member functions in C++.
+     */
+    virtual QCString trClassMethods()
+    {
+      return "Métodos de classe";
+    }
+
+    /*! Used as the header of a list of instance methods in Objective-C.
+     *  These are similar to public member functions in C++.
+     */
+    virtual QCString trInstanceMethods()
+    {
+      return "Métodos de instância";
+    }
+
+    /*! Used as the header of the member functions of an Objective-C class.
+     */
+    virtual QCString trMethodDocumentation()
+    {
+      return "Documentação do método";
+    }
+
+    /*! Used as the title of the design overview picture created for the
+     *  VHDL output.
+     */
+    virtual QCString trDesignOverview()
+    {
+      // I'm not sure how to accurately translate it
+      return "Visão geral do design";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.8.4
+//////////////////////////////////////////////////////////////////////////
+
+    /** old style UNO IDL services: implemented interfaces */
+    virtual QCString trInterfaces()
+    { return "Interfaces Exportadas"; }
+
+    /** old style UNO IDL services: inherited services */
+    virtual QCString trServices()
+    { return "Serviços Incluídos"; }
+
+    /** UNO IDL constant groups */
+    virtual QCString trConstantGroups()
+    { return "Grupos de Constantes"; }
+
+    /** UNO IDL constant groups */
+    virtual QCString trConstantGroupReference(const char *namespaceName)
+    {
+      QCString result = "Referência do grupo de constantes ";
+      result += namespaceName;
+      return result;
+    }
+
+    /** UNO IDL service page title */
+    virtual QCString trServiceReference(const char *sName)
+    {
+      QCString result = "Referência do serviço ";
+      result += sName;
+      return result;
+    }
+
+    /** UNO IDL singleton page title */
+    virtual QCString trSingletonReference(const char *sName)
+    {
+      QCString result = "Referência do Singleton ";
+      result += sName;
+      return result;
+    }
+
+    /** UNO IDL service page */
+    virtual QCString trServiceGeneratedFromFiles(bool single)
+    {
+      // single is true implies a single file
+      QCString result=(QCString)"A documentação para este serviço "
+                                "foi gerada a partir ";
+      if (single) {
+        result+="do seguinte ficheiro:";
+      } else {
+        result+="dos: seguintes ficheiros:";
+      }
+      return result;
+    }
+
+    /** UNO IDL singleton page */
+    virtual QCString trSingletonGeneratedFromFiles(bool single)
+    {
+      // single is true implies a single file
+      QCString result=(QCString)"A documentação para este singleton "
+                                "foi gerada a partir ";
+      if (single) {
+        result+="do seguinte ficheiro:";
+      } else {
+        result+="dos: seguintes ficheiros:";
+      }
+
+      return result;
+    }
+
+//////////////////////////////////////////////////////////////////////////
 };
 
 #endif
